//Bare arch test created by Michal Pytlos, July 2015

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#ifndef MICHAL_BARE_ARCH_H
#define MICHAL_BARE_ARCH_H

//#include <Testbed\Tests\michal_bare_arch_header.h> //gui
#include "michal_bare_arch_header.h" //non-gui

//non-input variables
float masonryDensity;
float archRadius;
float gamma;
float angleIncr;
bool fullGrav = false;
unsigned int gravityStep = 0;//gravity increment step
float noGravIncr;// number increments to reach full gravity
float tempGravity;
int myTimeStep;
float myTime;
bool run_simulation = true;
float saveTarget = 0;//in [m]; long file
float saveStep;
float saveTargetTime = 0.1;
float saveStepTime = 0.5; //[s]
float myDynamicThreshold;

//variables for ray cast
b2Vec2 loadPointIni;
b2Vec2 loadPoint;
b2Body* loadingBrick;
float loadingForce = 0;// in Newtons
bool castMyRay = false;
b2Vec2 point1Ini, point2Ini, point1, point2;
int hitBodyNo;
b2Vec2 localLoadPoint;
b2Vec2 loadCMassIni;
float loadAngleIni;
float relAngle;
b2Vec2 loadPointDispl;

MyContactListener contactListenerInstance; //instance of the contact listener

#ifdef GUI
class michal_bare_arch : public Test 
{
public:
    michal_bare_arch() {
#endif
#ifndef GUI
int main (int argc, char** argv)
{
	bool batch_input = false;
	if (argc >= 2)	{batch_input = true;}
	
	if(batch_input){
		//masonry
		span = atof(argv[1]);
		spanToRise = atof(argv[2]);
		ringThickness = atof(argv[3]);
		noBricks = atoi(argv[4]);
		myFriction = atof(argv[5]);
		loadingPos = atof(argv[6]);
		masonryUnitWeight = atof(argv[7]);
		//other
		testGrav = atof(argv[8]);
		skinThickness = atof(argv[9]);
		gravIncrTime = atof(argv[10]);
		maxLoadRate = atof(argv[11]);
		timeStepSize = atof(argv[12]);
		velocityIterations  = atoi(argv[13]);
		positionIterations  = atoi(argv[14]);
	}
	
	//build world
	b2Vec2 gravity(0.0, 0.0);
	b2World world(gravity);
#endif
#ifdef _WIN32
		start_cpuWindows = cpuTimer();
#endif
		noQBodies = noBricks + 2; //+2 for abutments
		noGravIncr = gravIncrTime/timeStepSize;
		saveStep = 0.02*ringThickness;
		masonryDensity = masonryUnitWeight*1000/abs(testGrav);//in kg/m2
		myDynamicThreshold = timeStepSize*abs(testGrav)/10;//[m/s] take into account that the lower the gravity the lower the velocities
		//Calculate arch geometry
			float rise = span/spanToRise;
			/*
			Eq (1): radius*sin(theta)=0.5*span 
			rearrange (1): Eq (2): radius=0.5*span/sin(theta)
			Eq (3): radius*(1-cos(theta))=rise 
			insert (2) into (3): Eq (4): (0.5*span/sin(theta))*(1-cos(theta))=rise 
			rearrange: 1-cos(theta)=sin(theta)*rise/(0.5*span)->Eq (5): cos(theta)+rise/(0.5*span)*sin(theta)=1

			Eq (6): a*cos(theta)+b*sin(theta)=R*cos(theta-alpha) [see sum of two trigonometric fucntions on mathcentre]
			from (5): a=1 and b=rise/(0.5*span) 
			*/
			float alpha = atan2(rise/(0.5*span),1);
			float myR = 1/cos(alpha);
			//Combine (6) with (5): Eq (7):  R*cos(theta-alpha)=1
			//Rearrange (7) (theta-alpha)= arccos(1/R)
			float theta = acos(1/myR)+alpha;
			archRadius = 0.5*span/sin(theta);//from (2)
			angleIncr = 2*theta/noBricks;
			gamma = myPI/2-theta;//for convenience

		//Adjust geometry to take into account skin thickness
			//angle difference to take into account skin thickness; 
				double angDiffB = atan2(skinThickness,archRadius);//bottom of the brick
				double angDiffT = atan2(skinThickness,archRadius+ringThickness);//top of the brick
			//adjusted radius difference
				float adjRadiusB = archRadius/cos(angDiffB);
				float adjRadiusT = (archRadius+ringThickness)/cos(angDiffT);	

		//Abutments
			//Right abutment
			//shape; adjusted for skin thickness			
			//x [global coords]
				float Ax0=adjRadiusB*cos(gamma-angDiffB);//left bottom corner
				float Ax1=adjRadiusB*cos(gamma-angDiffB)+0.5*ringThickness+ringThickness*cos(gamma-angDiffB);
				float Ax2=adjRadiusT*cos(gamma-angDiffT)+0.5*ringThickness;
				float Ax3=adjRadiusT*cos(gamma-angDiffT);
			//y [global coords]
				float Ay0=adjRadiusB*sin(gamma-angDiffB);
				float Ay1=adjRadiusB*sin(gamma-angDiffB);
				float Ay2=adjRadiusT*sin(gamma-angDiffT);
				float Ay3=adjRadiusT*sin(gamma-angDiffT);
				float vertXBlockR [4] = {Ax0,Ax1,Ax2,Ax3};//x coords of the vertices
				float vertYBlockR [4] = {Ay0,Ay1,Ay2,Ay3};// y coords of the vertices
				int abutmentID = 1;
				shapeQuad (vertXBlockR, vertYBlockR, abutmentID);

			//fixture
				b2BodyDef myBodyDef2;
				b2FixtureDef blockFixtureDef;
				blockFixtureDef.density = masonryDensity;
				blockFixtureDef.friction = myFriction;  
				blockFixtureDef.restitution = 0.0; 
				blockFixtureDef.shape = &polyShape;		
			//body definition
				myBodyDef2.type = b2_staticBody;
#ifdef GUI
				myQuad[abutmentID] = m_world->CreateBody(&myBodyDef2);
#endif
#ifndef GUI
				myQuad[abutmentID] = world.CreateBody(&myBodyDef2);
#endif
				myQuad[abutmentID]->CreateFixture(&blockFixtureDef);
			//Left abutment
				float vertXBlockL [4] = {-Ax0, -Ax3,-Ax2,-Ax1};// vertex 0 = right bottom corner
				float vertYBlockL [4] = {Ay0,Ay3,Ay2,Ay1};
				abutmentID = 0;
				shapeQuad (vertXBlockL, vertYBlockL, abutmentID);
				blockFixtureDef.shape = &polyShape;	
#ifdef GUI
				myQuad[abutmentID] = m_world->CreateBody(&myBodyDef2);
#endif
#ifndef GUI
				myQuad[abutmentID] = world.CreateBody(&myBodyDef2);
#endif
				myQuad[abutmentID]->CreateFixture(&blockFixtureDef);
				
		//Arch bricks
		b2BodyDef myBodyDef3;
		myBodyDef3.allowSleep=false;
		for (int i=0; i<noBricks; ++i){//start from left side
			//shape
			/* Alow for skin thickness:
			1. Left side of a block: add skin thickness in the normal direction (in direction +90 deg clockwise)
			2. Right side of a block: subtract skin thickness in the normal direction (in direction -90 deg clockwise)
			3. Also take into account mass difference. 
			*/

			//x [global coords]
				float Bx0=-adjRadiusB *cos(gamma+i*angleIncr+angDiffB);//x0 = left bottom corner
				float Bx1=-adjRadiusB *cos(gamma+(i+1)*angleIncr-angDiffB);//x1 = right bottom corner
				float Bx2=-adjRadiusT*cos(gamma+(i+1)*angleIncr-angDiffT);//x2 = top right
				float Bx3=-adjRadiusT*cos(gamma+i*angleIncr+angDiffT);//x3 = top left
			//y [global coords]
				float By0=adjRadiusB*sin(gamma+i*angleIncr+angDiffB);
				float By1=adjRadiusB*sin(gamma+(i+1)*angleIncr-angDiffB);
				float By2=adjRadiusT*sin(gamma+(i+1)*angleIncr-angDiffT);
				float By3=adjRadiusT*sin(gamma+i*angleIncr+angDiffT);

				float vertXBrick [4] = {Bx0,Bx1,Bx2,Bx3};//x coords of the vertices
				float vertYBrick [4] = {By0,By1,By2,By3};// y coords of the vertices
				int brickID = i+2;
				shapeQuad (vertXBrick, vertYBrick, brickID);
			//fixture
				b2FixtureDef brickFixtureDef;
				brickFixtureDef.density = masonryDensity;
				brickFixtureDef.friction = myFriction;  
				brickFixtureDef.restitution = 0.0; 
				brickFixtureDef.shape = &polyShape;	
			//body definition
				myBodyDef3.type = b2_dynamicBody;//change to static to inspect the initial block arrangement
#ifdef GUI
				myQuad[i+2] = m_world->CreateBody(&myBodyDef3);
#endif
#ifndef GUI
				myQuad[i+2] = world.CreateBody(&myBodyDef3);
#endif
				myQuad[i+2]->CreateFixture(&brickFixtureDef);
			}
#ifdef GUI
		m_world->SetGravity(b2Vec2(0.0, 0.0));
		m_world->SetContactListener(&contactListenerInstance); 
#endif
#ifndef GUI
		world.SetContactListener(&contactListenerInstance);
#endif
		//output to short file
		fileName=outputDir1;
		fileName.append("bareArch.txt");
		outputStream[0].open(fileName.c_str(), std::ios_base::app);
		//other outputs
		//Create all txt files; initialize streams for the output
		stringstream mySS;
		float identifierPart[14] = {span, spanToRise, ringThickness, noBricks, myFriction, abs(loadingPos), masonryUnitWeight, abs(testGrav), skinThickness, gravIncrTime, maxLoadRate, timeStepSize, velocityIterations, positionIterations};
		for (int i=0; i<14; ++i){//create file identifier
			mySS<<identifierPart[i]; 
			identifier.append(mySS.str());
			identifier.append("_");
			mySS.clear();
			mySS.str(string());
		}
		identifier.append("bareArch_");

		//append identifier to the identifiers.txt
		fileName=outputDir2;
		fileName.append("Identifiers.txt");
		outputStream[7].open(fileName.c_str(), std::ios_base::app);
		outputStream[7]<<endl<<identifier;
		outputStream[7].close();
		initalizeStreams(); //drawing files only
		//long results file
		fileName=outputDir1;
		fileName.append(identifier);
		fileName.append("results.txt");
		outputStream[1].open(fileName.c_str());
		outputStream[1]<<"span,spanToRise,ringThickness,noBricks,myFriction,loadingPos,masonryUnitWeight,testGrav,skinThickness,gravIncrTime,maxLoadRate,timeStepSize,velocityIterations,positionIterations"
						<<endl<<span<<","<<spanToRise<<","<<ringThickness<<","<<noBricks<<","<<myFriction<<","<<loadingPos<<","<<masonryUnitWeight
						<<","<<testGrav<<","<<skinThickness<<","<<gravIncrTime<<","<<maxLoadRate<<","<<timeStepSize<<","<<velocityIterations<<","<<positionIterations
						<<endl<<endl<<"time[s],tempGravity,load[kN],loadingPos,loadingBrickRot,normalLeftAbutment[kN],normalRightAbutment[kN]";

		saveQGeom();//save geometry of all quadrilaterals 
		cout<<"bare arch test"<<endl;
#ifdef GUI
    }//end test class

    void Step(Settings* settings)
    {
        Test::Step(settings);
#endif
#ifndef GUI 
	while (run_simulation){
		world.Step(timeStepSize, velocityIterations, positionIterations);
#endif

		myTime=myTimeStep*timeStepSize;
		++myTimeStep;
		myNormalForce[0]=myNormalImpulse[0]/timeStepSize;//left
		myNormalForce[1]=myNormalImpulse[1]/timeStepSize;//right
		myTangentForce[0]=myTangentImpulse[0]/timeStepSize;
		myTangentForce[1]=myTangentImpulse[1]/timeStepSize;

		//Incerementally apply the gravity
		if (!fullGrav) {
			++gravityStep;
			tempGravity = testGrav*gravityStep/noGravIncr;
#ifdef GUI
			m_world->SetGravity( b2Vec2(0,tempGravity) );//apply temp gravity
#endif
#ifndef GUI
			world.SetGravity( b2Vec2(0,tempGravity) );
#endif
			if (gravityStep>noGravIncr){fullGrav=true;};
		}
		
		if(fullGrav && !castMyRay){
			//cast ray to determine the point on the extrados corresponding to the loadingPos
			myRayCast rayCastInstance;
			point1Ini.x = loadingPos*span;
			point1Ini.y = archRadius+2*ringThickness;
			point2Ini.x = loadingPos*span;
			point2Ini.y = archRadius-2*ringThickness;
#ifdef GUI
			m_world->RayCast(&rayCastInstance, point1Ini, point2Ini);
#endif
#ifndef GUI
			world.RayCast(&rayCastInstance, point1Ini, point2Ini);
#endif
			//loop through all bodies hit by the ray -> find body with the lowest fraction (i.e. closest to the ray source)
			float minFraction = 1;
			for (int i = 0; i<rayCastInstance.myCount; ++i){
				if (rayCastInstance.m_fraction[i]<minFraction && rayCastInstance.m_fraction[i]>0){minFraction = rayCastInstance.m_fraction[i]; hitBodyNo = i;}
			}

			loadPointIni = rayCastInstance.m_point[hitBodyNo];
			loadingBrick = rayCastInstance.hitBody[hitBodyNo];
			loadCMassIni = loadingBrick->GetWorldCenter();
			loadAngleIni=loadingBrick->GetAngle();
			localLoadPoint = loadPointIni-loadCMassIni;

			castMyRay=true;
		}
	
		if (castMyRay){
			myRayCast rayCastInstance2;
			//take into brick linear movement
			point1=point1Ini+(loadingBrick->GetWorldCenter()-loadCMassIni);
			point2=point2Ini+(loadingBrick->GetWorldCenter()-loadCMassIni);
			//take into account brick rotation
			relAngle=loadingBrick->GetAngle()-loadAngleIni;
			b2Vec2 rotLocalLoadPoint = rotateVector(relAngle,localLoadPoint);
			point1 = point1+(rotLocalLoadPoint-localLoadPoint);
			point2 = point2+(rotLocalLoadPoint-localLoadPoint);
#ifdef GUI			
			m_world->RayCast(&rayCastInstance2, point1, point2);
#endif
#ifndef GUI
			world.RayCast(&rayCastInstance2, point1, point2);
#endif
			//loop through all bodies hit by the ray -> find body with the lowest fraction (i.e. closest to the ray source)
			float minFraction = 1;
			for (int i = 0; i<rayCastInstance2.myCount; ++i){
				if (rayCastInstance2.m_fraction[i]<minFraction && rayCastInstance2.m_fraction[i]>0){minFraction = rayCastInstance2.m_fraction[i]; hitBodyNo = i;}
			}

			loadPoint = rayCastInstance2.m_point[hitBodyNo];
			loadPointDispl=loadPoint-loadPointIni;

			//simple pseudo servo-control;
			//target: velocity is to be low and the load increment cannot be negative
			b2Vec2 loadVel = loadingBrick->GetLinearVelocity();
			float forceIncr = maxLoadRate*1000*timeStepSize;
			if (abs(loadVel.Length())>myDynamicThreshold){forceIncr=0;}//if the system behaves too dynamically do not increase the load
			loadingForce += forceIncr;//THIS SHOULD BE PER SECOND NOT PER TIME STEP TO_DO
			loadingBrick->ApplyForce(b2Vec2 (0,-loadingForce), loadPoint);
#ifdef GUI
			//mark loadingBrick and loadingPoint
			m_debugDraw.DrawCircle(loadPoint,0.05,b2Color(0.8f, 0.8f, 0.8f));//current
			m_debugDraw.DrawCircle(loadPointIni,0.05,b2Color(0.0f, 1.0f, 0.0f));//ini
			m_debugDraw.DrawCircle(loadingBrick->GetWorldCenter(),0.05,b2Color(0.0f, 1.0f, 0.0f));
			m_debugDraw.DrawSegment(point1,point2,b2Color(0.8f, 0.8f, 0.8f));// ray	current
			m_debugDraw.DrawSegment(point1Ini,point2Ini,b2Color(0.0f, 1.0f, 0.0f));// ray ini	

			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "loadingForce: %.0f", loadingForce);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "brickVel: %.4f", loadVel.Length());
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "loadIncr: %.0f", forceIncr);
#endif
		}

#ifdef GUI
		m_textLine += 30;
		m_debugDraw.DrawString(5, m_textLine, "myTime[s]: %.1f", myTime);	
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "tempGravity: %.3f", tempGravity);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "leftAbutmentNormalForce: %.0f", myNormalForce[0]);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "rightAbutmentNormalForce: %.0f", myNormalForce[1]);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "tempGravity: %.3f", tempGravity);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "loadingBrickDispl: %.3f", loadPointDispl.Length());
		
		//draw design brick arrangement
		for (int i=0; i<noBricks; ++i){//start from left side
			b2Vec2 myDrawVertices [4];
			myDrawVertices[0].Set(-archRadius*cos(gamma+i*angleIncr),archRadius*sin(gamma+i*angleIncr));
			myDrawVertices[1].Set(-archRadius*cos(gamma+(i+1)*angleIncr),archRadius*sin(gamma+(i+1)*angleIncr));
			myDrawVertices[2].Set(-(archRadius+ringThickness)*cos(gamma+(i+1)*angleIncr),(archRadius+ringThickness)*sin(gamma+(i+1)*angleIncr));
			myDrawVertices[3].Set(-(archRadius+ringThickness)*cos(gamma+i*angleIncr),(archRadius+ringThickness)*sin(gamma+i*angleIncr));
			m_debugDraw.DrawPolygon(myDrawVertices,4, b2Color(1.0f, 0.0f, 0.0f));
		}
#endif
		if ((loadPointDispl.Length()>saveTarget || myTime>saveTargetTime) && run_simulation){
			"time[s],load[kN],loadingPos,loadingBrickRot";
				outputStream[1]<<endl<<myTime<<","<<-tempGravity<<","<<loadingForce/1000<<","<<loadPointDispl.Length()<<","<<relAngle*RADTODEG<<","<<myNormalForce[0]/1000<<","<<myNormalForce[1]/1000;
				if (loadPointDispl.Length()>saveTarget){
					saveTarget+=saveStep;
					saveQSim();//save simulation data (x,y,rotation) of all quadrilaterals to the relevant output files
					++noTS;
				}
				if (myTime>saveTargetTime){saveTargetTime+=saveStepTime;}
		} 	

		if (loadPointDispl.Length()>ringThickness){run_simulation=false;}
		//saveData and end test
		if (!run_simulation){
#ifdef _WIN32
			stop_cpuWindows = cpuTimer();
			cpuTimeWindows = stop_cpuWindows-start_cpuWindows;
#endif
			outputStream[0]<<endl<<loadingForce/1000<<","<<myTime<<","<<loadPointDispl.Length()<<","<<cpuTimeWindows<<","
				<<span<<","<<spanToRise<<","<<ringThickness<<","<<noBricks<<","<<myFriction<<","<<loadingPos<<","<<masonryUnitWeight<<","
				<<testGrav<<","<<skinThickness<<","<<gravIncrTime<<","<<maxLoadRate<<","<<timeStepSize<<","<<velocityIterations<<","<<positionIterations;
			outputStream[0].close();
			saveMeta();
			outputStream[2].close();
		}
    }
#ifdef GUI
    static Test* Create()
    {
        return new michal_bare_arch;
    }
#endif
#ifndef GUI
	cout<<"end of the test"<<endl;
	return(0);
#endif
};

#endif

