//Biaxial compression test created by Michal Pytlos

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#ifndef MICHAL_BIAXIAL
#define MICHAL_BIAXIAL

//#include <Testbed\Tests\michal_biaxial_header.h> //comment in non-gui
#include "michal_biaxial_header.h" //uncomment in non-gui

#ifdef GUI
	extern bool batch_input;
	extern char** g_argv;
	extern int g_argc;
#endif

//variables used in the test
	float timeStepSize = 1.0/60.0;
	int velocityIterations = 8;
	int positionIterations = 3;	
	float sampleWidth = 0; // average width measured by the rays
	float sampleHeight;//sample height measured during the test
	float startHeight; //sample height at the start of the test
	float axialStrain;
	bool castMyRay, startTest, doOnce, changeFri;
	float sumParticleForceT, sumParticleForceB;
	unsigned int myTimeStep = 0;
	unsigned int gravityStep = 0;//gravity reduction step
	int frictionStep = 0;
	b2Fixture* tempFixture;
	float tempGravity, tempFri;
	bool zeroGrav, levelSample, weighSample;//bools for gravity reduction
	int weighStep = 0;
	unsigned int pressureStep = 1;//apply confining pressure incrementaly in x steps
	float sumPosLeft = 0; //sum of x positions of points on the bodies hit by the ray; this variable is used to calculate the average width
	float sumPosRight = 0;
	float aveAxialForce;
	int saveFreqG; //save geometry data every 'saveFreqG' time steps; this is automatically set so the final number of recorded steps is ~100
	int rayNo; //number of rays; this is automatically set so there are 3 rays per boundary particle on average
	float iniWidth; //initial sample width = number of particles to sample width * average particle diameter
	float iniHeight; //initial sample height
	float sMassAve = 0; //sample mass averaged over 1000 time steps
	float sMass[1000]; //sample mass 
	float massMin = FLT_MAX;//maximum value for the integer value
	float massMax = 0;
	int removedPBodies = 0;
	b2Vec2 posVec[maxNoP]; //pos vector for particles
	float depoHeight = 1.35; //height of the area for particles creation
	bool PIDsuccess;

MyContactListener contactListenerInstance; //instance of the contact listener

#ifdef GUI
class michal_biaxial : public Test 
{
public:	
    michal_biaxial() {

		if(batch_input){//if there is batch input initialize confining pressure and internal angle of friction
			confPressure = atof(g_argv[1]); 
			partIniFri = atof(g_argv[2]);
			partFri = atof(g_argv[3]); 
			boundFri = atof(g_argv[4]); 
			partDensity = atof(g_argv[5]); 
			capVel = atof(g_argv[6]); 
			noPToWidth = atof(g_argv[7]);
			randSeed = atoi(g_argv[8]);
			depoGrav = atof(g_argv[9]);
			HRHigh = atof(g_argv[10]);
			strainLim = atof(g_argv[11]);
			K_p = atof(g_argv[12]);
			K_iFric = atof(g_argv[13]);
		}
#endif
#ifndef GUI
int main (int argc, char** argv)
{
	bool run_simulation = true;
	bool batch_input = false;
	if (argc >= 2)	{batch_input = true;}
	
	if(batch_input){
		confPressure = atof(argv[1]); 
		partIniFri = atof(argv[2]);
		partFri = atof(argv[3]); 
		boundFri = atof(argv[4]); 
		partDensity = atof(argv[5]); 
		capVel = atof(argv[6]); 
		noPToWidth = atof(argv[7]);
		randSeed = atoi(argv[8]);
		timeStepSize = atof(argv[9]);
		velocityIterations = atoi(argv[10]);
		depoGrav = atof(argv[11]);
		HRHigh = atof(argv[12]);
		strainLim = atof(argv[13]);
		K_p = atof(argv[14]);
		K_iFric = atof(argv[15]);
	}
	
	//build world
	b2Vec2 gravity(0.0, 0.0);
	b2World world(gravity);
#endif

		iniWidth = noPToWidth*(circRL+circRH); //initial sample width = number of particles to sample width * average particle diameter
		iniHeight = iniWidth*iniHtoW; //initial sample height
		noPBodies = noPToWidth * noPToWidth * iniHtoW * 1.14 /((1+HRHigh)/2);
		rayNo = 3*iniHeight/(circRL*2);
		srand(randSeed); //set seed for the RandomFloat() 
		tempGravity = depoGrav;
		tempFri = partIniFri;
		//time
		time_t rawtime; 
		char myDT[80];//simulation date and time
		time(&rawtime);//get current time
		struct tm * timeinfo = localtime (&rawtime);
		strftime(myDT, 80, "%d/%m/%Y_%H:%M:%S", timeinfo);

		//Define body
			b2BodyDef myBodyDef;
			
		//Bottom static boundary
			//shape
			float vertXBB [2] = {-iniWidth*1.2/2, iniWidth*1.2/2};//x coords of the vertices
			float vertYBB [2] = {0, 0};// y coords of the vertices
			const int bBoundID = 0;
			shapeEdge (vertXBB, vertYBB, bBoundID);

			//fixture 
			b2FixtureDef boundFixtureDef;
			boundFixtureDef.friction = boundFri; 
			boundFixtureDef.restitution = 0.0; 
			boundFixtureDef.shape = &edgeShape;
            
			//create body and add fixture to it 
			myBodyDef.type = b2_staticBody;
#ifdef GUI
			myEdge[bBoundID] = m_world->CreateBody(&myBodyDef);
#endif
#ifndef GUI
			myEdge[bBoundID] = world.CreateBody(&myBodyDef);
#endif
			myEdge[bBoundID]->CreateFixture(&boundFixtureDef);

		//Left static boundary; this boundary is deleted after sample deposition;
			boundFixtureDef.friction = 0.0; 
			float vertXLB [2] = {-iniWidth/2, -iniWidth/2};//x coords of the vertices
			float vertYLB [2] = {0, iniHeight*2.0};// y coords of the vertices
			const int lBoundID = 1;
			shapeEdge (vertXLB, vertYLB, lBoundID);
			boundFixtureDef.shape = &edgeShape;
#ifdef GUI
			myEdge[lBoundID] = m_world->CreateBody(&myBodyDef);
#endif
#ifndef GUI
			myEdge[lBoundID] = world.CreateBody(&myBodyDef);
#endif
			myEdge[lBoundID]->CreateFixture(&boundFixtureDef);

		//Right static boundary; this boundary is deleted after sample deposition;
			boundFixtureDef.friction = 0.0; 
			float vertXRB [2] = {iniWidth/2, iniWidth/2};
			float vertYRB [2] = {0, iniHeight*2.0};
			const int rBoundID = 2;
			shapeEdge (vertXRB, vertYRB, rBoundID);
			boundFixtureDef.shape = &edgeShape;
#ifdef GUI
			myEdge[rBoundID] = m_world->CreateBody(&myBodyDef);
#endif
#ifndef GUI
			myEdge[rBoundID] = world.CreateBody(&myBodyDef);
#endif
			myEdge[rBoundID]->CreateFixture(&boundFixtureDef);

		//Dodecagon particles
			//fixture
			b2BodyDef myBodyDef2;
			b2FixtureDef particleFixtureDef;
			particleFixtureDef.density = partDensity;
			particleFixtureDef.friction = partIniFri;  
			particleFixtureDef.restitution = particleRestitution; 

			//body definition
			myBodyDef2.type = b2_dynamicBody;
			myBodyDef2.allowSleep = true;

			for (int i = 0; i<noPBodies; ++i)
			{
				//position
				bool foundPos = false;
				int posNo = 0; //number of posisiton trials
				while (!foundPos)// loop unitl not overlapping position is found
				{
					foundPos = true;
					if (posNo>1000){depoHeight+=0.01; posNo=0;}//if cannot suitable position increase the creation area
					posVec[i].x=RandomFloat(-iniWidth/2+circRH, iniWidth/2-circRH);
					posVec[i].y=RandomFloat(5*circRH, iniHeight*depoHeight+5*circRH);
					for(int j = 0; j<i; ++j)
					{
						b2Vec2 tempVec = posVec[i];//get a vector that is egual to the difference between j and i
						tempVec -= posVec[j];	
						if (tempVec.Length()<circRH*2){foundPos=false;}
					}
					++posNo;
				}
				myBodyDef2.position = posVec[i]; 
				myBodyDef2.angle = DEGTORAD*RandomFloat(0.0f, 359.0f);

				//shape
				shapeDodecagon(circRL,circRH,irrF,HRHigh,i);
				particleFixtureDef.shape = &polyShape;

				//create body
#ifdef GUI
				myParticle[i] = m_world->CreateBody(&myBodyDef2);
#endif
#ifndef GUI
				myParticle[i] = world.CreateBody(&myBodyDef2);
#endif
				myParticle[i]->CreateFixture(&particleFixtureDef);
			}
#ifdef GUI
	 m_world->SetGravity( b2Vec2(0,depoGrav) );
	 m_world->SetContactListener(&contactListenerInstance); 
#endif
#ifndef GUI
	world.SetGravity( b2Vec2(0,depoGrav) );
	world.SetContactListener(&contactListenerInstance);
#endif
	createIdentifier(confPressure, partIniFri, partFri, boundFri, partDensity, abs(capVel), noPToWidth, randSeed, timeStepSize, velocityIterations, abs(depoGrav), HRHigh);
	initalizeStreams();

	outputStream[0]<<"confPressure, partIniFri, partFri, boundFri, partDensity, capVel, noPToWidth, randSeed, timeStepSize, velocityIterations, noPBodies, depoHeight, depoGrav, HRHigh, Date"<<endl;
	outputStream[0]<<confPressure<<","<<partIniFri<<","<<partFri<<","<<boundFri<<","<<partDensity<<","<<capVel<<","<<noPToWidth<<","<<randSeed<<","<<timeStepSize
		<<","<<velocityIterations<<","<<noPBodies<<","<<depoHeight<<","<<depoGrav<<","<<HRHigh<<","<<myDT<<endl<<endl;
	outputStream[0]<<"SampleHeight,SampleWidth,AxialForce,SampleMass,removedPBodies,massDiff"<<endl;

	saveEGeom();//save geometry of all edges	
	savePGeom();//save geometry of all particles

	for (int i=0; i<noPBodies; ++i) {bodyTotRotP[i]=0;}//intialize bodyTotRot
	cout<<"biaxial test"<<endl;
#ifdef GUI
	}//end test class

    void Step(Settings* settings)
	{
        //run the default physics and rendering
        Test::Step(settings);
#endif
#ifndef GUI 
	while (run_simulation)
	{
		world.Step(timeStepSize, velocityIterations, positionIterations);
#endif
		//Check if particles are sleeping
		bool sleeping = true;
		for (int i=0; i<noPBodies; ++i) {if (myParticle[i]->IsAwake()) {sleeping = false;}}
#ifdef GUI
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "noPBodies: %.i", noPBodies);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "Friction: %.3f", tempFri);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "Gravity: %.3f", tempGravity);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "Number of H rays: %.i", rayNo);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "Sample mass ave: %.0f", sMassAve);
		m_debugDraw.DrawSegment(b2Vec2(-iniWidth/2,iniHeight),b2Vec2(iniWidth/2,iniHeight),b2Color(1.0f, 0.1f, 0.1f));//top bound
		m_debugDraw.DrawSegment(b2Vec2(-iniWidth/2,0),b2Vec2(-iniWidth/2,iniHeight),b2Color(1.0f, 0.0f, 0.0f));//temp left bound
		m_debugDraw.DrawSegment(b2Vec2(iniWidth/2,0),b2Vec2(iniWidth/2,iniHeight),b2Color(1.0f, 0.0f, 0.0f));//temp right bound
#endif
		//Level top surface if all particles are sleeping
		if (sleeping && !levelSample){
			for (int i=0; i<noPBodies; ++i) {
				b2Vec2 myPos = myParticle[i]->GetPosition();
				if (myPos.y>iniHeight-circRH){
					myParticle[i]->SetTransform( b2Vec2(myPos.x, myPos.y*1.2), myParticle[i]->GetAngle()); //if a body is above intial sample height teleport it up ...
					myParticle[i]->SetType(b2_staticBody);// ... and make it static 
					++removedPBodies;
				}
				myParticle[i]->SetSleepingAllowed(false);//particles are not allowed to sleep;
			}
			levelSample = true;
		}

		//Wait for the weight measurement to steady before saving the value
		if (levelSample && ! weighSample){
			sumParticleForceB = 0;
			for (int i=0; i<myCounter2; ++i) {sumParticleForceB +=pow(timeStepSize,-1)*particleImpulseB[i];}
			sMass[weighStep] = -sumParticleForceB/depoGrav;
#ifdef GUI			
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Sample mass: %.0f", sMass[weighStep]);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Weigh step: %.i", weighStep);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Sample mass min: %.0f", massMin);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Sample mass max: %.0f", massMax);
#endif
			++ weighStep;
			if (weighStep==1000){
				massMin = FLT_MAX;//maximum value for the integer value
				massMax = 0;
				float massSum = 0;
				for (int i=0; i<weighStep; ++i){if(sMass[i]<massMin){massMin=sMass[i];}}//min mass in the range
				for (int i=0; i<weighStep; ++i){if(sMass[i]>massMax){massMax=sMass[i];}}//max mass in the range
				for (int i=0; i<weighStep; ++i){massSum += sMass[i];}
				sMassAve = massSum/weighStep;
				if ((massMax-massMin)/sMassAve < 0.005){weighSample=true;}//save weight 
				else {weighStep = 0;}
			}
		}

		//Decrementally zero the gravity
		if (weighSample && !zeroGrav) {
			++gravityStep;
			tempGravity = depoGrav*(noGravDecr-gravityStep)/noGravDecr;
#ifdef GUI
			m_world->SetGravity( b2Vec2(0,tempGravity) );//reduce the gravity
#endif
#ifndef GUI
			world.SetGravity( b2Vec2(0,tempGravity) );//reduce the gravity
#endif
			for (int i=0; i<noPBodies; ++i) {myParticle[i]->SetAwake(true);}//awake all bodies

			if (gravityStep==noGravDecr){zeroGrav=true;};
		} 

		//Destroy side the boundaries and create the top cap
		if (!castMyRay && zeroGrav){
		
			destroyEdge(1);//destroy left boundary
			destroyEdge(2);//destroy right boundary

			//Create top kinematic boundary
			float vertXTB [4] = {-iniWidth*1.2/2, iniWidth*1.2/2, iniWidth*1.2/2, -iniWidth*1.2/2};//x coords of the vertices
			float vertYTB [4] = {-capH/2, -capH/2, capH/2, capH/2};// y coords of the vertices
			const int tBoundID = 0;//ID
			shapeQuad(vertXTB, vertYTB, tBoundID);
			b2BodyDef myBodyDef;
			b2FixtureDef boundFixtureDef;
			boundFixtureDef.friction = boundFri; 
			boundFixtureDef.shape = &polyShape;
			myBodyDef.type = b2_kinematicBody;
			myBodyDef.allowSleep=false; //this plate is not allowed to sleep !
			myBodyDef.position.Set(0.0, iniHeight+capH/2);
#ifdef GUI
			myQuad[tBoundID] = m_world->CreateBody(&myBodyDef);
#endif
#ifndef GUI
			myQuad[tBoundID] = world.CreateBody(&myBodyDef);
#endif
			myQuad[tBoundID]->CreateFixture(&boundFixtureDef);
			saveQGeom();//save geometry of all quadrilaterals

			castMyRay = true;
		}

		//Apply confining pressure
		if (castMyRay)
		{
			++myTimeStep;

			b2Vec2 platePos = myQuad[0]->GetPosition();
			sampleHeight = platePos.y-capH/2;

			float hConfForce = (pressureStep/incrNo)*confPressure*sampleHeight/rayNo;
			float vConfForce = (pressureStep/incrNo)*confPressure*sampleWidth;

			// add impulses from all relevant particles to get force
			sumParticleForceT = 0;//reset sum
			sumParticleForceB = 0;
			for (int i=0; i<myCounter3; ++i) {sumParticleForceT +=pow(timeStepSize,-1)*particleImpulseT[i];}
			for (int i=0; i<myCounter2; ++i) {sumParticleForceB +=pow(timeStepSize,-1)*particleImpulseB[i];}
			aveAxialForce = (sumParticleForceT + sumParticleForceB)/2;
#ifdef GUI
			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "Boundary friction: %.1f", boundFri);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Particle density: %.1f", partDensity);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Confining pressure: %.0f", confPressure);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Cap velocity: %.4f", capVel);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Number of particles to width ratio: %.0f", noPToWidth);
			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "Sample height: %.5f", sampleHeight);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "sampleWidth: %.3f", sampleWidth);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Axial force: %.1f", aveAxialForce);
			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "V conf force: %.1f", vConfForce);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "H conf force: %.1f", hConfForce);
			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "Time Step: %.i", myTimeStep);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "Start test: %s ", (startTest)?"true":"false"); 
#endif			
			//confining pressure in the horizontal direction
			for (int i=0; i<rayNo; ++i)
			{
				myRayCast rayCastInstL, rayCastInstR; //rayCastInstL = Left; rayCastInstR = Right; 
				b2Vec2 point1L, point2L, point1R, point2R;
				//horizontal left
				point1L.x = -iniWidth*0.8;
				point2L.x = -1.0;
				point1L.y = point2L.y = (0.5+i)*(sampleHeight/(rayNo));
				//horizontal right 
				point1R.x = iniWidth*0.8;
				point2R.x = 1.0;
				point1R.y = point2R.y = point1L.y;
#ifdef GUI				
				m_world->RayCast(&rayCastInstL, point1L, point2L);
				m_world->RayCast(&rayCastInstR, point1R, point2R);
				m_debugDraw.DrawSegment(point1L,point2L,b2Color(0.1f, 0.1f, 0.1f));
				m_debugDraw.DrawSegment(point1R,point2R,b2Color(0.1f, 0.1f, 0.1f));
#endif
#ifndef GUI
				world.RayCast(&rayCastInstL, point1L, point2L);
				world.RayCast(&rayCastInstR, point1R, point2R);
#endif			
				float minFractionL = 1;
				float minFractionR = 1;
				int hitBodyNoL, hitBodyNoR;

				//find body closest to the source of the ray (with min fraction)
				for (int i = 0; i<rayCastInstL.myCount; ++i){ //Left side
					if (rayCastInstL.m_fraction[i]<minFractionL && rayCastInstL.m_fraction[i]>0 && rayCastInstL.hitBody[i]->GetType() == b2_dynamicBody){minFractionL = rayCastInstL.m_fraction[i]; hitBodyNoL = i;}
				}
				for (int i = 0; i<rayCastInstR.myCount; ++i){ //Right side
					if (rayCastInstR.m_fraction[i]<minFractionR && rayCastInstR.m_fraction[i]>0 && rayCastInstR.hitBody[i]->GetType() == b2_dynamicBody){minFractionR = rayCastInstR.m_fraction[i]; hitBodyNoR = i;}
				}

				//apply confining force AND mark the body with red circle 
				if (rayCastInstL.m_hit){ //Left side
#ifdef GUI
					m_debugDraw.DrawCircle(rayCastInstL.hitBody[hitBodyNoL]->GetWorldCenter(), 0.05, b2Color(1.0f, 0.0f, 0.0f));
#endif
					rayCastInstL.hitBody[hitBodyNoL]->ApplyForce( b2Vec2(hConfForce,0), rayCastInstL.hitBody[hitBodyNoL]->GetWorldCenter() );
				}
				if (rayCastInstR.m_hit){ //Right side
#ifdef GUI
					m_debugDraw.DrawCircle(rayCastInstR.hitBody[hitBodyNoR]->GetWorldCenter(), 0.05, b2Color(1.0f, 0.0f, 0.0f));
#endif
					rayCastInstR.hitBody[hitBodyNoR]->ApplyForce( b2Vec2(-hConfForce,0), rayCastInstR.hitBody[hitBodyNoR]->GetWorldCenter() );
				}
				
				//calculate average sample width - part 1
				sumPosLeft += rayCastInstL.m_point[hitBodyNoL].x;
				sumPosRight += rayCastInstR.m_point[hitBodyNoR].x;
			}

			//calculate average sample width - part 2
			sampleWidth = (sumPosRight-sumPosLeft)/rayNo;
			sumPosLeft = 0;
			sumPosRight = 0;

			//Axial confining pressure applied via a servo-controlled top cap
			if (!startTest)
			{
#ifdef GUI
				m_textLine += 30;
				m_debugDraw.DrawString(5, m_textLine, "PID step: %.i", PIDstep);
				m_textLine += 15;
				m_debugDraw.DrawString(5, m_textLine, "Proportional error: %.3f", errorAve);
				m_textLine += 15;
				m_debugDraw.DrawString(5, m_textLine, "Integral error: %.3f", errorInteg);
				m_textLine += 15;
				m_debugDraw.DrawString(5, m_textLine, "Derivative error: %.3f", errorDeriv);
				m_textLine += 15;
				m_debugDraw.DrawString(5, m_textLine, "Cap conf vel: %.6f", capConfVel);
				m_textLine += 15;
				m_debugDraw.DrawString(5, m_textLine, "Pressure step: %.i", pressureStep);
				m_textLine += 15;
				m_debugDraw.DrawString(5, m_textLine, "Friction step: %.i", frictionStep);
				m_textLine += 15;
				m_debugDraw.DrawString(5, m_textLine, "K_i: %.5f", K_i);
#endif
				PIDsuccess=false;
				if (vConfForce>0.01){PIDcontrol(&PIDsuccess, vConfForce, timeStepSize, aveAxialForce, myTimeStep);}//vConfForce=0 in the first step
				if (PIDsuccess && pressureStep<incrNo){++pressureStep;}
				if (PIDsuccess && pressureStep==incrNo && !changeFri){
					changeFri = true; 
					K_i=K_iFric;//change integral gain value
					for (int i=0; i<noPBodies; ++i) {myParticle[i]->SetSleepingAllowed(true);}
				}
			} 

			//Change particle friction; do this gradually
			if (changeFri && !startTest){
				if (PIDsuccess){
					++frictionStep;
					tempFri = partIniFri + (partFri-partIniFri)*(frictionStep/noFriSteps);
					for (int i=0; i<noPBodies; ++i) {
						tempFixture=myParticle[i]->GetFixtureList();
						tempFixture->SetFriction(tempFri);//this does not change friction of the existing contacts; need to reset the contacts for this to kick in
						myParticle[i]->SetAwake(true);//awake all bodies; need this for the contact listener to kick in
						myReset=true;//reset the contact
					}
				}
				if (frictionStep==noFriSteps) {startTest = true;}
			}
		}

		//Start biaxial compression stage
		if (startTest && !doOnce) {
			myQuad[0]->SetLinearVelocity(b2Vec2(0, capVel));
			startHeight = sampleHeight;
			saveFreqG = abs(((startHeight*0.1)/(timeStepSize*capVel))/100);//0.1 = test finished at 10% axial strain; 100 = aim for 100 recorded steps
			outputStream[0]<<sampleHeight<<","<<sampleWidth<<","<<aveAxialForce<<","<<sMassAve<<","<<removedPBodies<<","<<massMax-massMin<<endl;//save output - first row with additional info
			doOnce = true;
			cout<<"biaxial compression stage started"<<endl;
		}
		//save output and update bodyTotRot 
		if (startTest && myTimeStep % 1 == 0 && doOnce) {
			outputStream[0]<<sampleHeight<<","<<sampleWidth<<","<<aveAxialForce<<endl;//save output
			
			axialStrain = (startHeight - sampleHeight)/startHeight*100;
#ifdef GUI
			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "axial strain: %.3f", axialStrain);
			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "save freq: %.i", saveFreqG);
#endif
			for (int i=0; i<noPBodies; ++i){//bodyTotRot
				float rotIncr = timeStepSize * myParticle[i]->GetAngularVelocity();
				bodyTotRotP[i] += rotIncr;
			}
		}
		//save geometry 
		if (startTest && myTimeStep % saveFreqG == 0 && doOnce) {
			saveQSim();//save simulation data (x,y,rotation) of all quadrilaterals to the relevant output files
			savePSim();//save simulation data (x,y,rotation, total rotation) of all dodecagon particles to the relevant output files
			cout<<"axial strain: "<<axialStrain<<endl;
			++noTS;
		}
		
		myCounter2 = 0; //reset the contact listener counter (for the next time step) 
		myCounter3 = 0;

		if (axialStrain>strainLim && doOnce){
			saveMeta();
			exit(0);//end test
		}
	}

#ifdef GUI
    static Test* Create()
    {
        return new michal_biaxial;
    } 
#endif
};

#endif