//Backfilled arch test with load applied via a loadinbg beam; created by Michal Pytlos, August 2015; header file

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#ifndef MICHAL_BACKFILLED_ARCHLB_HEADER_H
#define MICHAL_BACKFILLED_ARCHLB_HEADER_H

//#define GUI
#include <iomanip> /*fixed,showpoint,setprecision()*/
#include <iostream>
#include <sstream>
#include <fstream>
#include <math.h>       /* sin */
#include<ctime>
#ifdef _WIN32
	#include <windows.h>
#endif

#define myPI 3.14159265358979323846f
#ifndef DEGTORAD /* this macro changes degrees to radians and vice-versa */
#define DEGTORAD 0.0174532925199432957f
#define RADTODEG 57.295779513082320876f
#endif

#ifdef GUI
	#include <Testbed\Tests\michal_shape_definition.h>
#endif
#ifndef GUI
	#include "michal_shape_definition_no_gui.h"
	using namespace std;
#endif

//input
	//arch
float span = 20.0;//arch span
float spanToRise = 4;
float ringThickness = 1.2;
int noBricks = 25;
float myFriction = 0.6; //coefficient of friction for bricks and abutments
float masonryUnitWeight = 20;// in kN/m2
	//soil
float partIniFri = 0.2;
float circRH = 0.5; //particle circum radius high
float particleUnitWeight = 26.50;// in kN/m2
float fillHeight = 8;
float lBoundOffset = 0.25;//left bound; must be positive
float rBoundOffset = 0.5;//this number is multiplied by the arch span; zero offset = at the outer face of the abutment; must be positive
	//loading 
float loadingPos = -0.25; //unit = arch span; relative to the center of the arch
float beamWidth = 5; 
float beamVel = -0.01;
float displLim = 0.5; // [m]; end the test at this value of displacement of the loading beam
	//other
float testGrav = -1.0;
unsigned int randSeed = 1; 
float timeStepSize = 1.0/60.0;
int velocityIterations = 8;

//paramaters to be kept constant
	//soil
float partFri = 0.6;//test friction
const float particleRestitution = 0.0;
const float irrF = 100.0; //irregularity factor
const float HRHigh = 1.0;//height ratio high
float depoGrav = -0.1;//gravity during deposition
float depoHeight = 1.00; //height of the area for particles creation; this gets multiplied bu the fillHeight
	//other
float gravIncrTime = 10.0;//reach test gravity in this many seconds 
int positionIterations = 3;	
const float noFriSteps = 100;// number of friction change steps
float skinThickness = 0.00070; //aim for approx 0.0070 IF b2_polygonRadius = (2.0f * b2_linearSlop)
float targetVel = 0.01;//target loading brick movement rate

//parameters calculated automatically
float  masonryDensity;
float partDensity;
float circRL; //particle circum radius low
float abutmentHoriz; //length of the extra horizontal part of the abutment

//bodies
const int noCBodies = 0;
const int noEBodies = 3;
int noPBodies;
int noQBodies;
int noTS = 0;// number of geometry entries to the outout files
float bodyTotRotP[maxNoP];//total rotation of a given particle
const int maxNoBricks = 1000;
b2Body* myQuad[maxNoBricks+3]; //[0]=left abutment; [1]=right abutment; [2]=brick most to the left ...; [n]=loading beam 
b2Body* myEdge[noEBodies];
b2Body* myParticle[maxNoP];

//measure CPU time on Windows
double start_cpuWindows;
double stop_cpuWindows;
double measuredCPUTime;
//measure CPY time on UNIX // clock() returns CPU time only on UNIX
clock_t start_cpuUNIX;
clock_t elapsed_cpuUNIX;

//output
	std::ofstream outputStream[14];//[0]=testResults short file; [1]long file; [2]metaData; [3]=QGeometry; [4]=QGlobalX; [5]=QGlobalY; [6]=QRotation; 
	//[7]=PGeometry; [8]=PGlobalX; [9]=PGlobalY; [10]=PRotation; [11]=PTotRot; [12]=EGeometry;[13]=identifiers;
#ifdef _WIN32	
	string outputDir1 ="../../../../backfilledArchLB/"; //windows only
	string outputDir2 ="../../../../backfilledArchLB/drawings/"; //windows only
#endif
#ifdef __unix__
	string outputDir1 ="./MyResults/"; 
	string outputDir2 ="./MyResults/drawings/"; 
#endif
	string fileName;
	string identifier;

void initalizeStreams(){//initialize streams for the outputs [drawing files only]
 
	string fileType[11] = {"metaData","QGeometry","QGlobalX","QGlobalY","QRotation","PGeometry","PGlobalX","PGlobalY","PRotation","PTotRot","EGeometry"}; //this is asyName[] in retaining wall

	for (int i=0; i<11; ++i){//streams [2] to [12]
		fileName=outputDir2;
		fileName.append(identifier);
		fileName.append(fileType[i]);
		fileName.append(".txt");
		outputStream[i+2].open(fileName.c_str());
	}
}

void saveQGeom (){//save geometry of all quadrilaterals to QGeometry.txt;
	for (int i=0; i<noQBodies; ++i){
		b2Vec2 localCenterOfMassQ = myQuad[i]->GetLocalCenter();
		outputStream[3]<<x0Q[i]<<","<<x1Q[i]<<","<<x2Q[i]<<","<<x3Q[i]<<","<<localCenterOfMassQ.x<<","<<y0Q[i]<<","<<y1Q[i]<<","<<y2Q[i]<<","<<y3Q[i]<<","<<localCenterOfMassQ.y<<endl;
	}
}

void saveEGeom(){//save geometry of all edges to EGeometry.txt;
	for (int i=0; i<noEBodies; ++i){
		outputStream[12]<<x0E[i]<<","<<x1E[i]<<","<<y0E[i]<<","<<y1E[i]<<endl;
	}
}

void savePGeom(){//save geometry of all dodecagon particles to PGeometry.txt;
	for (int i=0; i<noPBodies; ++i){
		b2Vec2 localCenterOfMassP = myParticle[i]->GetLocalCenter();
		outputStream[7]<<x0P[i]<<","<<x1P[i]<<","<<x2P[i]<<","<<x3P[i]<<","<<x4P[i]<<","<<x5P[i]<<","<<x6P[i]<<","<<x7P[i]<<","<<x8P[i]<<","<<x9P[i]<<","<<x10P[i]<<","<<x11P[i]<<","<<localCenterOfMassP.x<<","<<y0P[i]<<","<<y1P[i]<<","<<y2P[i]<<","<<y3P[i]<<","<<y4P[i]<<","<<y5P[i]<<","<<y6P[i]<<","<<y7P[i]<<","<<y8P[i]<<","<<y9P[i]<<","<<y10P[i]<<","<<y11P[i]<<","<<localCenterOfMassP.y<<endl;
	}
}

void saveQSim(){//save simulation data (x,y,rotation) of all quadrilaterals to the relevant output files
	for (int i=0; i<noQBodies; ++i){	
		b2Vec2 bodyPosQ=myQuad[i]->GetPosition();
		float bodyRotQ=myQuad[i]->GetAngle(); //in radians
		outputStream[4]<<bodyPosQ.x<<",";
		outputStream[5]<<bodyPosQ.y<<",";
		outputStream[6]<<bodyRotQ<<",";
	}
	outputStream[4]<<endl;
	outputStream[5]<<endl;
	outputStream[6]<<endl;
}

void savePSim(){
	for (int i=0; i<noPBodies; ++i){
		b2Vec2 bodyPosP = myParticle[i]->GetPosition();
		float bodyRotP = myParticle[i]->GetAngle();
		outputStream[8]<<bodyPosP.x<<",";
		outputStream[9]<<bodyPosP.y<<",";
		outputStream[10]<<bodyRotP<<",";
		outputStream[11]<<bodyTotRotP[i]<<",";
	}
	outputStream[8]<<endl;
	outputStream[9]<<endl;
	outputStream[10]<<endl;
	outputStream[11]<<endl;
}

void saveMeta(){//save meta data to metaData.txt
	outputStream[2]<<noQBodies<<","<<noPBodies<<","<<noCBodies<<","<<noEBodies<<","<<noTS;//write info to metaData file
}

//cast ray class
class myRayCast : public b2RayCastCallback 
{
public:

	myRayCast()
	{
		m_hit = false;
		myCount=0;
	}

	float32 ReportFixture(	b2Fixture* fixture, const b2Vec2& point,const b2Vec2& normal, float32 fraction){
		hitBody[myCount] = fixture->GetBody();
		//report only if hit body is a brick
		bool isBrick = false;
		for (int i=0; i<noBricks; ++i){
			if (hitBody[myCount]==myQuad[i+2]){isBrick=true;}
		}
		if (isBrick){
			m_fraction[myCount] = fraction;
			m_point[myCount] = point; //point on the body hit by the ray; in world coordinates
			m_hit = true;
			++myCount;
		}
		
		return 1.0;//don't clip the ray and continue
	}
	float m_fraction[maxNoP];
	b2Body* hitBody[maxNoP];
	int myCount;
	bool m_hit;
	b2Vec2 m_point[maxNoP];
};

//contact listener
float particleImpulse[maxNoP];
int myCounter2;//used instead of 'i' to reduce CPU time spent on the loop. Counts particles in contact with the surface in a given time step 
int numPoints; //number of collision contact points 
bool myReset;//bool to reset the contact

class MyContactListener : public b2ContactListener
{
public:
 
	void PostSolve (b2Contact* contact,const b2ContactImpulse* impulse)
	{
		//identify colliding bodies
		const b2Body* bodyA = contact->GetFixtureA()->GetBody(); 
		const b2Body* bodyB = contact->GetFixtureB()->GetBody(); 

		//get force by adding all particles in contact with the loading beam
		for (int i=0; i<noPBodies;++i){
			if (((bodyA == myParticle[i]) || (bodyB == myParticle[i])) && ((bodyA == myQuad[noBricks+2]) || (bodyB == myQuad[noBricks+2]))){	
				//check number of contact points
				numPoints = contact->GetManifold()->pointCount;

				if (numPoints == 2) {particleImpulse[myCounter2] = (impulse->normalImpulses[0] + impulse->normalImpulses[1]);}

				//if there is only one contact point, ignore normalImpulses[1]
				if (numPoints == 1) {particleImpulse[myCounter2] = (impulse->normalImpulses[0]);}

				++ myCounter2;
			}
		}
		if (myReset) {contact->ResetFriction();}//reset the contact for the new friction value to kick in
	}
};


b2Vec2 rotateVector (float angle, const b2Vec2& v) //angles in radians; rotate anticlockwise
{
	float s, c;
	s = sinf(angle);
	c = cosf(angle);

	//create rotation matrix	|r11 r12|
	//							|r21 r22|
		
	float r11, r12, r21, r22;
	r11 = c;
	r12 = -s;
	r21 = s;
	r22 = c;

	//multiply by column vector |r11 r12|	|x|
	//							|r21 r22|	|y|
		
	b2Vec2 v_g;
	v_g.x = r11*v.x + r12*v.y;
	v_g.y = r21*v.x + r22*v.y;

	return v_g;
};
#ifdef _WIN32
double cpuTimer() //Function to measure cpu time on Windows OS
{ 
	//structures to store times recorder by GetProcessTimes()
	FILETIME createTime; 
    FILETIME exitTime;
    FILETIME kernelTime;
    FILETIME userTime;

	GetProcessTimes (GetCurrentProcess( ), &createTime, &exitTime, &kernelTime, &userTime );//get timing	
	SYSTEMTIME userSystemTime;//structure to store user time
	FileTimeToSystemTime( &userTime, &userSystemTime );//covert times

	//return value in seconds
	return (double)userSystemTime.wHour * 3600.0 + (double)userSystemTime.wMinute * 60.0 + (double)userSystemTime.wSecond + (double)userSystemTime.wMilliseconds / 1000.0;
};
#endif
#endif

