//Backfilled arch test created by Michal Pytlos, July 2015

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#ifndef MICHAL_BACKFILLED_ARCH_H
#define MICHAL_BACKFILLED_ARCH_H

//#include <Testbed\Tests\michal_backfilled_arch_header.h> //gui
#include "michal_backfilled_arch_header.h" //non-gui

//non-input variables
b2Vec2 posVec[maxNoP]; //pos vector for particles
bool levelSample, gravStage, changedFri, bricksDynamic, castMyRay, startTest;
int removedPBodies = 0;
int frictionStep = 0;
float tempFriction;
b2Fixture* tempFixture;
float leftBoundPos;
float rightBoundPos;
float bottomBoundPos;
float archRadius;
float myGamma;
float angleIncr;
unsigned int gravityStep = 0;//gravity increment step
float noGravIncr;// number increments to reach test gravity
float tempGravity;
int myTimeStep;
float myTime;
bool run_simulation = true;
float saveTargetDispl = 0;//in [m]; long file
float saveStepDispl;
float displayTargetTime = 0;//in [s]
float displayStepTime = 25;//in [s]
float myDynamicThreshold;
float backfillMass=0;
float backfillVolume;
float backfillUnitWeight;

//variables for ray cast
b2Vec2 loadPointIni;
b2Vec2 loadPoint;
b2Body* loadingBrick;
float loadingForce = 0;// in Newtons
b2Vec2 point1Ini, point2Ini, point1, point2;
int hitBodyNo;
b2Vec2 localLoadPoint;
b2Vec2 loadCMassIni;
float loadAngleIni;
float relAngle;
b2Vec2 loadPointDispl;

MyContactListener contactListenerInstance; //instance of the contact listener

#ifdef GUI
class michal_backfilled_arch : public Test 
{
public:
    michal_backfilled_arch() {
#endif
#ifndef GUI
int main (int argc, char** argv)
{
	bool batch_input = false;
	if (argc >= 2)	{batch_input = true;}
	
	if(batch_input){
		//arch
		span = atof(argv[1]);
		spanToRise = atof(argv[2]);
		ringThickness = atof(argv[3]);
		noBricks = atoi(argv[4]);
		myFriction = atof(argv[5]);
		loadingPos = atof(argv[6]);
		masonryUnitWeight = atof(argv[7]);
		//soil
		partIniFri = atof(argv[8]);
		circRH = atof(argv[9]);
		particleUnitWeight = atof(argv[10]);
		fillHeight = atof(argv[11]);
		lBoundOffset = abs(atof(argv[12]));
		rBoundOffset = abs(atof(argv[13]));
		//other
		testGrav = atof(argv[14]);
		maxLoadRate = atof(argv[15]);
		randSeed = atoi(argv[16]);
		timeStepSize = atof(argv[17]);
		velocityIterations = atoi(argv[18]);
	}
	
	//build world
	b2Vec2 gravity(0.0, 0.0);
	b2World world(gravity);
#endif
#ifdef _WIN32
		start_cpuWindows = cpuTimer();
#endif
#ifdef __unix__
		start_cpuUNIX = clock();
#endif
		noQBodies = noBricks + 2; //+2 for abutments
		noGravIncr = gravIncrTime/timeStepSize;
		saveStepDispl = 0.02*ringThickness;//0.02 = 50 geometry entries
		masonryDensity = masonryUnitWeight*1000/abs(testGrav);//in kg/m2
		partDensity = particleUnitWeight*1000/abs(testGrav);//in kg/m2
		circRL = circRH; //one size for all particles
		myDynamicThreshold=timeStepSize*abs(testGrav)/10;//[m/s] take into account that the lower the gravity the lower the velocities
		abutmentHoriz = 0.5*ringThickness;
		srand(randSeed); //set seed for the RandomFloat() 

		//Calculate arch geometry
			float rise = span/spanToRise;
			/*
			Eq (1): radius*sin(theta)=0.5*span 
			rearrange (1): Eq (2): radius=0.5*span/sin(theta)
			Eq (3): radius*(1-cos(theta))=rise 
			insert (2) into (3): Eq (4): (0.5*span/sin(theta))*(1-cos(theta))=rise 
			rearrange: 1-cos(theta)=sin(theta)*rise/(0.5*span)->Eq (5): cos(theta)+rise/(0.5*span)*sin(theta)=1

			Eq (6): a*cos(theta)+b*sin(theta)=R*cos(theta-alpha) [see sum of two trigonometric fucntions on mathcentre]
			from (5): a=1 and b=rise/(0.5*span) 
			*/
			float alpha = atan2(rise/(0.5*span),1);
			float myR = 1/cos(alpha);
			//Combine (6) with (5): Eq (7):  R*cos(theta-alpha)=1
			//Rearrange (7) (theta-alpha)= arccos(1/R)
			float theta = acos(1/myR)+alpha;
			archRadius = 0.5*span/sin(theta);//from (2)
			angleIncr = 2*theta/noBricks;
			myGamma = myPI/2-theta;//for convenience

		//Adjust geometry to take into account skin thickness
			//angle difference to take into account skin thickness; 
				double angDiffB = atan2(skinThickness,archRadius);//bottom of the brick
				double angDiffT = atan2(skinThickness,archRadius+ringThickness);//top of the brick
			//adjusted radius difference
				float adjRadiusB = archRadius/cos(angDiffB);
				float adjRadiusT = (archRadius+ringThickness)/cos(angDiffT);
		//Caculate number of particles
			float modelWidth = (1+lBoundOffset+rBoundOffset)*span+2*abutmentHoriz;
			float modelHeight = fillHeight;
			float abutmentHeight=(archRadius+ringThickness)*sin(myGamma)-archRadius*sin(myGamma);
			float abutmentTriangleBase=(archRadius+ringThickness)*cos(myGamma)-archRadius*cos(myGamma);;
			float abutmentArea=abutmentTriangleBase*abutmentHeight/2+abutmentHeight*abutmentHoriz;//triangle + rectangular
			float archArea1 = myPI*pow(archRadius+ringThickness,2)*((2*theta)/(2*myPI));//part 1
			float archArea2 = (archRadius+ringThickness)*sin(theta)*(archRadius+ringThickness)*cos(theta);//part 2
			float archArea = archArea1-archArea2;
			backfillVolume = modelWidth*modelHeight-archArea-2*abutmentArea;
			float avePartDiam = (circRH+circRL)/2*2;//particle average diameter
			float partVolFactor = (1/avePartDiam)*(1/avePartDiam)*1.35 /((1+HRHigh)/2);
			noPBodies = backfillVolume*partVolFactor;//total number of particles to be created
			int noPBodiesZone1 = (lBoundOffset*span+abutmentHoriz)*fillHeight*partVolFactor;// zone 1: to the left of the arch
			int noPBodiesZone2 = (rBoundOffset*span+abutmentHoriz)*fillHeight*partVolFactor;//zone 2: to the right of the arch
			int noPBodiesZone3 = noPBodies - noPBodiesZone1 - noPBodiesZone2;//zone 3: directly above the arch
			int noPBodiesZone3_1 = noPBodiesZone3/3.5;// zone 3 part 1: 0 to 1/4 span above the arch
			int noPBodiesZone3_3 = noPBodiesZone3_1;// zone 3 part 3: 3/4 to 1 span above the arch
			int noPBodiesZone3_2 = noPBodies-noPBodiesZone1-noPBodiesZone2-noPBodiesZone3_1-noPBodiesZone3_3;//this variable is not used anywhere

		//Abutments
			//Right abutment
			//shape; adjusted for skin thickness			
			//x [global coords]
				float Ax0=adjRadiusB*cos(myGamma-angDiffB);//left bottom corner
				float Ax1=adjRadiusB*cos(myGamma-angDiffB)+abutmentHoriz+ringThickness*cos(myGamma-angDiffB);
				float Ax2=adjRadiusT*cos(myGamma-angDiffT)+abutmentHoriz;
				float Ax3=adjRadiusT*cos(myGamma-angDiffT);
			//y [global coords]
				float Ay0=adjRadiusB*sin(myGamma-angDiffB);
				float Ay1=adjRadiusB*sin(myGamma-angDiffB);
				float Ay2=adjRadiusT*sin(myGamma-angDiffT);
				float Ay3=adjRadiusT*sin(myGamma-angDiffT);
				float vertXBlockR [4] = {Ax0,Ax1,Ax2,Ax3};//x coords of the vertices
				float vertYBlockR [4] = {Ay0,Ay1,Ay2,Ay3};// y coords of the vertices
				int abutmentID = 1;
				shapeQuad (vertXBlockR, vertYBlockR, abutmentID);

			//fixture
				b2BodyDef myBodyDef2;
				b2FixtureDef blockFixtureDef;
				blockFixtureDef.density = masonryDensity;
				blockFixtureDef.friction = myFriction;  
				blockFixtureDef.restitution = 0.0; 
				blockFixtureDef.shape = &polyShape;		
			//body definition
				myBodyDef2.type = b2_staticBody;
#ifdef GUI
				myQuad[abutmentID] = m_world->CreateBody(&myBodyDef2);
#endif
#ifndef GUI
				myQuad[abutmentID] = world.CreateBody(&myBodyDef2);
#endif
				myQuad[abutmentID]->CreateFixture(&blockFixtureDef);
			//Left abutment
				float vertXBlockL [4] = {-Ax0, -Ax3,-Ax2,-Ax1};// vertex 0 = right bottom corner
				float vertYBlockL [4] = {Ay0,Ay3,Ay2,Ay1};
				abutmentID = 0;
				shapeQuad (vertXBlockL, vertYBlockL, abutmentID);
				blockFixtureDef.shape = &polyShape;	
#ifdef GUI
				myQuad[abutmentID] = m_world->CreateBody(&myBodyDef2);
#endif
#ifndef GUI
				myQuad[abutmentID] = world.CreateBody(&myBodyDef2);
#endif
				myQuad[abutmentID]->CreateFixture(&blockFixtureDef);
				
		//Arch bricks
		b2BodyDef myBodyDef3;
		myBodyDef3.allowSleep=false;
		for (int i=0; i<noBricks; ++i){//start from left side
			//shape
			/* Alow for skin thickness:
			1. Left side of a block: add skin thickness in the normal direction (in direction +90 deg clockwise)
			2. Right side of a block: subtract skin thickness in the normal direction (in direction -90 deg clockwise)
			3. Also take into account mass difference. 
			*/

			//x [global coords]
				float Bx0=-adjRadiusB *cos(myGamma+i*angleIncr+angDiffB);//x0 = left bottom corner
				float Bx1=-adjRadiusB *cos(myGamma+(i+1)*angleIncr-angDiffB);//x1 = right bottom corner
				float Bx2=-adjRadiusT*cos(myGamma+(i+1)*angleIncr-angDiffT);//x2 = top right
				float Bx3=-adjRadiusT*cos(myGamma+i*angleIncr+angDiffT);//x3 = top left
			//y [global coords]
				float By0=adjRadiusB*sin(myGamma+i*angleIncr+angDiffB);
				float By1=adjRadiusB*sin(myGamma+(i+1)*angleIncr-angDiffB);
				float By2=adjRadiusT*sin(myGamma+(i+1)*angleIncr-angDiffT);
				float By3=adjRadiusT*sin(myGamma+i*angleIncr+angDiffT);

				float vertXBrick [4] = {Bx0,Bx1,Bx2,Bx3};//x coords of the vertices
				float vertYBrick [4] = {By0,By1,By2,By3};// y coords of the vertices
				int brickID = i+2;
				shapeQuad (vertXBrick, vertYBrick, brickID);
			//fixture
				b2FixtureDef brickFixtureDef;
				brickFixtureDef.density = masonryDensity;
				brickFixtureDef.friction = myFriction;  
				brickFixtureDef.restitution = 0.0; 
				brickFixtureDef.shape = &polyShape;	
			//body definition
				myBodyDef3.type = b2_staticBody;//change to static to inspect the initial block arrangement
#ifdef GUI
				myQuad[i+2] = m_world->CreateBody(&myBodyDef3);
#endif
#ifndef GUI
				myQuad[i+2] = world.CreateBody(&myBodyDef3);
#endif
				myQuad[i+2]->CreateFixture(&brickFixtureDef);
			}
		//Boundaries
		b2BodyDef myBodyDef;
		leftBoundPos = -Ax1-lBoundOffset*span;
		rightBoundPos = Ax1+rBoundOffset*span;
		bottomBoundPos = Ay0-2*skinThickness;
		//Bottom static boundary
			//shape
			float vertXBB [2] = {leftBoundPos, rightBoundPos};//x coords of the vertices
			float vertYBB [2] = {bottomBoundPos, bottomBoundPos};// y coords of the vertices
			const int bBoundID = 0;
			shapeEdge (vertXBB, vertYBB, bBoundID);
			//fixture 
			b2FixtureDef boundFixtureDef;
			boundFixtureDef.friction = 0.0; 
			boundFixtureDef.restitution = 0.0; 
			boundFixtureDef.shape = &edgeShape;
			//create body and add fixture to it 
			myBodyDef.type = b2_staticBody;
#ifdef GUI
			myEdge[bBoundID] = m_world->CreateBody(&myBodyDef);
#endif
#ifndef GUI
			myEdge[bBoundID] = world.CreateBody(&myBodyDef);
#endif
			myEdge[bBoundID]->CreateFixture(&boundFixtureDef);

		//Left boundary
			//shape
			float vertXBL [2] = {leftBoundPos-2*skinThickness, leftBoundPos-2*skinThickness};//x coords of the vertices
			float vertYBL [2] = {Ay0, Ay0+fillHeight*3.0};// y coords of the vertices
			const int lBoundID = 1;
			shapeEdge (vertXBL, vertYBL, lBoundID);
			//create body
#ifdef GUI
			myEdge[lBoundID] = m_world->CreateBody(&myBodyDef);
#endif
#ifndef GUI
			myEdge[lBoundID] = world.CreateBody(&myBodyDef);
#endif
			myEdge[lBoundID]->CreateFixture(&boundFixtureDef);

		//Right boundary
			//shape
			float vertXBR [2] = {rightBoundPos+2*skinThickness, rightBoundPos+2*skinThickness};//x coords of the vertices
			float vertYBR [2] = {Ay0, Ay0+fillHeight*3.0};// y coords of the vertices
			const int rBoundID = 2;
			shapeEdge (vertXBR, vertYBR, rBoundID);
			//create body
#ifdef GUI
			myEdge[rBoundID] = m_world->CreateBody(&myBodyDef);
#endif
#ifndef GUI
			myEdge[rBoundID] = world.CreateBody(&myBodyDef);
#endif
			myEdge[rBoundID]->CreateFixture(&boundFixtureDef);

	//Dodecagon particles
			//fixture
			b2BodyDef myBodyDef4;
			b2FixtureDef particleFixtureDef;
			particleFixtureDef.density = partDensity;
			particleFixtureDef.friction = partIniFri;  
			particleFixtureDef.restitution = particleRestitution; 

			//body definition
			myBodyDef4.type = b2_dynamicBody;
			myBodyDef4.allowSleep = true;

			
			for (int i = 0; i<noPBodies; ++i)
			{
				//position
				bool foundPos = false;
				bool zone1 = false;
				bool zone2 = false;
				int posNo = 0; //number of positon trials 
				float depoHeightTemp = depoHeight;

				while (!foundPos)// loop unitl not overlapping position is found
				{
					foundPos = true;
					if (posNo>100){depoHeightTemp+=0.1; posNo=0;}//if cannot find suitable position increase the creation area

					//horizontal position
					if (i<noPBodiesZone1){posVec[i].x=RandomFloat(leftBoundPos+circRH, -Ax3); zone1=true;}//first create all particles in the zone 1: to the left of the arch 
					else if (i<noPBodiesZone1+noPBodiesZone2){posVec[i].x=RandomFloat(Ax3, rightBoundPos-circRH); zone2=true;}//then create all particles in the zone 2: to the right of the arch
					else if (i<noPBodiesZone1+noPBodiesZone2+noPBodiesZone3_1){posVec[i].x=RandomFloat(-Ax3,-Ax3+(span/4));}//then create particles in the zone 3_1
					else if	(i<noPBodiesZone1+noPBodiesZone2+noPBodiesZone3_1+noPBodiesZone3_3){posVec[i].x=RandomFloat(Ax3-(span/4),Ax3);}//then create particles in the zone 3_3
					else {posVec[i].x=RandomFloat(-Ax3+(span/4),Ax3-(span/4));}//finally create particles in the zone 3_2: 1/4 to 3/4 span above the arch 

					//vertical position
					float lowPos; //lowest allowable position (depends on the horizontal position)
					if (zone1){
						if (posVec[i].x>-Ax1){lowPos=Ay2;}//this is above the flat part of the abutments
						else{lowPos = archRadius*sin(myGamma);}
						posVec[i].y=RandomFloat(lowPos+5*circRH, bottomBoundPos+fillHeight*depoHeightTemp);
					}
					else if (zone2){
						if (posVec[i].x<Ax1){lowPos=Ay2;}//this is above the flat part of the abutments
						else{lowPos = archRadius*sin(myGamma);}
						posVec[i].y=RandomFloat(lowPos+5*circRH, bottomBoundPos+fillHeight*depoHeightTemp);
					}

					else {//zone 3
						lowPos=sqrt(pow(archRadius+ringThickness,2)-pow(posVec[i].x,2));//from Pythagoras
						posVec[i].y=RandomFloat(lowPos+5*circRH, bottomBoundPos+fillHeight*depoHeightTemp);
					}

					//check overlap with existing particles
					for(int j = 0; j<i; ++j)
					{
						b2Vec2 tempVec = posVec[i];//get a vector that is egual to the difference between j and i
						tempVec -= posVec[j];	
						if (tempVec.Length()<circRH*2){foundPos=false;}
					}
					++posNo;
				}
				myBodyDef4.position = posVec[i]; 
				myBodyDef4.angle = DEGTORAD*RandomFloat(0.0f, 359.0f);

				//shape
				shapeDodecagon(circRL,circRH,irrF,HRHigh,i);
				particleFixtureDef.shape = &polyShape;

				//create body
#ifdef GUI
				myParticle[i] = m_world->CreateBody(&myBodyDef4);
#endif
#ifndef GUI
				myParticle[i] = world.CreateBody(&myBodyDef4);
#endif
				myParticle[i]->CreateFixture(&particleFixtureDef);
			}   
			//END CREATE PARTICLES
#ifdef GUI
		m_world->SetGravity(b2Vec2(0.0, depoGrav));
		m_world->SetContactListener(&contactListenerInstance); 
#endif
#ifndef GUI
		world.SetGravity(b2Vec2(0.0, depoGrav));
		world.SetContactListener(&contactListenerInstance);
#endif
		//output to short file
		fileName=outputDir1;
		fileName.append("backfilledArch.txt");
		outputStream[0].open(fileName.c_str(), std::ios_base::app);
		//other outputs
		//Create all txt files; initialize streams for the output
		stringstream mySS;
		float identifierPart[18] = {span, spanToRise, ringThickness, noBricks, myFriction, abs(loadingPos), masonryUnitWeight, 
			partIniFri, circRH, particleUnitWeight, fillHeight, lBoundOffset, rBoundOffset, 
			abs(testGrav), maxLoadRate, randSeed, timeStepSize, velocityIterations};
		for (int i=0; i<18; ++i){//create file identifier
			mySS<<identifierPart[i]; 
			identifier.append(mySS.str());
			identifier.append("_");
			mySS.clear();
			mySS.str(string());
		}
		identifier.append("backfilledArch_");

		//append identifier to the identifiers.txt
		fileName=outputDir2;
		fileName.append("Identifiers.txt");
		outputStream[13].open(fileName.c_str(), std::ios_base::app);
		outputStream[13]<<endl<<identifier;
		outputStream[13].close();
		initalizeStreams(); //drawing files only
		//long results file
		fileName=outputDir1;
		fileName.append(identifier);
		fileName.append("results.txt");
		outputStream[1].open(fileName.c_str());
		outputStream[1]<<"span, spanToRise, ringThickness, noBricks, myFriction, loadingPos, masonryUnitWeight, partIniFri, circRH, particleUnitWeight, fillHeight, lBoundOffset, rBoundOffset, testGrav, maxLoadRate, randSeed, timeStepSize, velocityIterations, masonryDensity, particleDensity"
						<<endl<<span<<","<<spanToRise<<","<<ringThickness<<","<<noBricks<<","<<myFriction<<","<<loadingPos<<","<<masonryUnitWeight
						<<","<<partIniFri<<","<<circRH<<","<<particleUnitWeight<<","<<fillHeight<<","<<lBoundOffset<<","<<rBoundOffset
						<<","<<testGrav <<","<<maxLoadRate<<","<<randSeed<<","<<timeStepSize<<","<<velocityIterations<<","<<masonryDensity<<","<<partDensity<<endl<<endl;

		saveQGeom();//save geometry of all quadrilaterals 
		saveEGeom();//save geometry of all edges	
		savePGeom();//save geometry of all particles
		for (int i=0; i<noPBodies; ++i) {bodyTotRotP[i]=0;}//intialize bodyTotRot
		cout<<endl<<"Backfiled arch test: "<<identifier<<endl<<endl<<"Number of soil particles in the model: "<<noPBodies<<endl;
#ifdef GUI
    }//end test class

    void Step(Settings* settings)
    {
        Test::Step(settings);
#endif
#ifndef GUI 
	while (run_simulation){
		world.Step(timeStepSize, velocityIterations, positionIterations);
#endif

		myTime=myTimeStep*timeStepSize;
		++myTimeStep;

		//console output
		if (myTime>displayTargetTime){
#ifdef _WIN32
			stop_cpuWindows = cpuTimer();
			measuredCPUTime = stop_cpuWindows-start_cpuWindows;
#endif
#ifdef __unix__
			elapsed_cpuUNIX = clock() - start_cpuUNIX;
			measuredCPUTime = ((double)elapsed_cpuUNIX)/CLOCKS_PER_SEC;
#endif
			cout<<fixed<<showpoint<<setprecision(1)<<"Test time: "<<displayTargetTime<<"s. CPU time: "<<measuredCPUTime<<"s. ";
			displayTargetTime += displayStepTime;
			if (!changedFri){cout<<"Backfill deposition stage."<<endl;}
			else {cout<<"Load: "<<loadingForce/1000<<"kN. Displacement: "<<loadPointDispl.Length()<<"m."<<endl;}
		}
		//DEL//myNormalForce[0]=myNormalImpulse[0]/timeStepSize;//left
		//DEL//myNormalForce[1]=myNormalImpulse[1]/timeStepSize;//right
		//DEL//myTangentForce[0]=myTangentImpulse[0]/timeStepSize;
		//DEL//myTangentForce[1]=myTangentImpulse[1]/timeStepSize;

		//Check if particles are sleeping
		bool sleeping = true;
		for (int i=0; i<noPBodies; ++i) {if (myParticle[i]->IsAwake()) {sleeping = false;}}

		//Level top surface if all particles are sleeping or the time limit is exceeded //TO_DO: improve the time limit 
		if ((sleeping || myTime>100.0)&& !levelSample){

			for (int i=0; i<noPBodies; ++i) {
				b2Vec2 myPos = myParticle[i]->GetPosition();
				if (myPos.y>bottomBoundPos+fillHeight-circRH){
					myParticle[i]->SetTransform( b2Vec2(myPos.x, myPos.y*1.2), myParticle[i]->GetAngle()); //if a body is above intial sample height teleport it up ...
					myParticle[i]->SetType(b2_staticBody);// ... and make it static 
					++removedPBodies;
				}
				else {backfillMass += myParticle[i]->GetMass(); }//add mass to the total mass of the backfill
				myParticle[i]->SetSleepingAllowed(false);//particles are not allowed to sleep;
			}
			levelSample = true;
		}

		//Incerementally apply the gravity
		if (levelSample && !gravStage) {
			float gravDiff = testGrav-depoGrav;
			++gravityStep;
			tempGravity = depoGrav+gravDiff*gravityStep/noGravIncr;
#ifdef GUI
			m_world->SetGravity( b2Vec2(0,tempGravity) );//apply temp gravity
#endif
#ifndef GUI
			world.SetGravity( b2Vec2(0,tempGravity) );
#endif
			if (gravityStep>noGravIncr){gravStage=true;};
		}

		//Change particle friction; do this gradually
		if (gravStage && !changedFri){
			++frictionStep;
			float friDiff = partFri-partIniFri;
			tempFriction = partIniFri + friDiff*(frictionStep/noFriSteps);
			for (int i=0; i<noPBodies; ++i) {
				tempFixture=myParticle[i]->GetFixtureList();
				tempFixture->SetFriction(tempFriction);//this does not change friction of the existing contacts; need to reset the contacts for this to kick in
				myParticle[i]->SetAwake(true);//awake all bodies; need this for the contact listener to kick in
				myReset=true;//reset the contact
			}		
			if (frictionStep==noFriSteps) {changedFri = true;}
		}

		//Change voussoirs to dynamic bodies
		if (changedFri && !bricksDynamic){
			for (int i=0; i<noBricks; ++i){myQuad[i+2]->SetType(b2_dynamicBody);}
			bricksDynamic=true;
		}

		//Cast ray to determine the point on the extrados corresponding to the loadingPos
		if(bricksDynamic && !castMyRay){
			myRayCast rayCastInstance;
			point1Ini.x = loadingPos*span;
			point1Ini.y = archRadius+2*ringThickness;
			point2Ini.x = loadingPos*span;
			point2Ini.y = archRadius-2*ringThickness;
#ifdef GUI
			m_world->RayCast(&rayCastInstance, point1Ini, point2Ini);
#endif
#ifndef GUI
			world.RayCast(&rayCastInstance, point1Ini, point2Ini);
#endif
			//loop through all bodies hit by the ray -> find body with the lowest fraction (i.e. closest to the ray source)
			float minFraction = 1;
			for (int i = 0; i<rayCastInstance.myCount; ++i){
				if (rayCastInstance.m_fraction[i]<minFraction && rayCastInstance.m_fraction[i]>0){minFraction = rayCastInstance.m_fraction[i]; hitBodyNo = i;}
			}
			loadPointIni = rayCastInstance.m_point[hitBodyNo];
			loadingBrick = rayCastInstance.hitBody[hitBodyNo];
			loadCMassIni = loadingBrick->GetWorldCenter();
			loadAngleIni=loadingBrick->GetAngle();
			localLoadPoint = loadPointIni-loadCMassIni;

			castMyRay=true;
			startTest=true;
			//Save data
			backfillUnitWeight=backfillMass*abs(testGrav)/(backfillVolume*1000);//in kN/m2
			outputStream[1]<<"time[s],load[kN],loadPosDispl,noPBodies,removedPBodies,backfillUnitWeight[kN/m2]"
				<<endl<<myTime<<","<<loadingForce/1000<<","<<loadPointDispl.Length()<<","<<noPBodies<<","<<removedPBodies<<","<<backfillUnitWeight;//save output 
		}
	
		if (castMyRay){
			myRayCast rayCastInstance2;
			//take into account brick linear movement
			point1=point1Ini+(loadingBrick->GetWorldCenter()-loadCMassIni);
			point2=point2Ini+(loadingBrick->GetWorldCenter()-loadCMassIni);
			//take into account brick rotation
			relAngle=loadingBrick->GetAngle()-loadAngleIni;
			b2Vec2 rotLocalLoadPoint = rotateVector(relAngle,localLoadPoint);
			point1 = point1+(rotLocalLoadPoint-localLoadPoint);
			point2 = point2+(rotLocalLoadPoint-localLoadPoint);
#ifdef GUI			
			m_world->RayCast(&rayCastInstance2, point1, point2);
#endif
#ifndef GUI
			world.RayCast(&rayCastInstance2, point1, point2);
#endif
			//loop through all bodies hit by the ray -> find body with the lowest fraction (i.e. closest to the ray source)
			float minFraction = 1;
			for (int i = 0; i<rayCastInstance2.myCount; ++i){
				if (rayCastInstance2.m_fraction[i]<minFraction && rayCastInstance2.m_fraction[i]>0){minFraction = rayCastInstance2.m_fraction[i]; hitBodyNo = i;}
			}

			loadPoint = rayCastInstance2.m_point[hitBodyNo];
			loadPointDispl=loadPoint-loadPointIni;

			//simple pseudo servo-control;
			//target: velocity is to be low and the load increment cannot be negative
			b2Vec2 loadVel = loadingBrick->GetLinearVelocity();
			float forceIncr = maxLoadRate*1000*timeStepSize;
			if (abs(loadVel.Length())>myDynamicThreshold){forceIncr=0;} //if the system behaves too dynamically do not increase the load
			loadingForce += forceIncr;//THIS SHOULD BE PER SECOND NOT PER TIME STEP TO_DO
			loadingBrick->ApplyForce(b2Vec2 (0,-loadingForce), loadPoint);//APPLY FORCE
#ifdef GUI
			//mark loadingBrick and loadingPoint
			m_debugDraw.DrawCircle(loadPoint,0.05,b2Color(0.8f, 0.8f, 0.8f));//current
			m_debugDraw.DrawCircle(loadPointIni,0.05,b2Color(0.0f, 1.0f, 0.0f));//ini
			m_debugDraw.DrawCircle(loadingBrick->GetWorldCenter(),0.05,b2Color(0.0f, 1.0f, 0.0f));
			m_debugDraw.DrawSegment(point1,point2,b2Color(0.8f, 0.8f, 0.8f));// ray	current
			m_debugDraw.DrawSegment(point1Ini,point2Ini,b2Color(0.0f, 1.0f, 0.0f));// ray ini
			m_debugDraw.DrawSegment(b2Vec2(leftBoundPos,bottomBoundPos+fillHeight),b2Vec2(rightBoundPos,bottomBoundPos+fillHeight),b2Color(0.0f, 1.0f, 0.0f));// fill height	

			m_textLine += 30;
			m_debugDraw.DrawString(5, m_textLine, "loadingForce[kN]: %.1f", loadingForce/1000);
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "brickVel: %.4f", loadVel.Length());
			m_textLine += 15;
			m_debugDraw.DrawString(5, m_textLine, "loadIncr: %.0f", forceIncr);
#endif
		}//END if(castMyRay)

#ifdef GUI
		m_textLine += 30;
		m_debugDraw.DrawString(5, m_textLine, "myTime[s]: %.1f", myTime);	
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "tempGravity: %.3f", tempGravity);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "tempFriction: %.3f", tempFriction);
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "loadingBrickDispl: %.3f", loadPointDispl.Length());
		m_textLine += 15;
		m_debugDraw.DrawString(5, m_textLine, "noPBodies: %.i", noPBodies);
		
		//draw design brick arrangement
		for (int i=0; i<noBricks; ++i){//start from left side
			b2Vec2 myDrawVertices [4];
			myDrawVertices[0].Set(-archRadius*cos(myGamma+i*angleIncr),archRadius*sin(myGamma+i*angleIncr));
			myDrawVertices[1].Set(-archRadius*cos(myGamma+(i+1)*angleIncr),archRadius*sin(myGamma+(i+1)*angleIncr));
			myDrawVertices[2].Set(-(archRadius+ringThickness)*cos(myGamma+(i+1)*angleIncr),(archRadius+ringThickness)*sin(myGamma+(i+1)*angleIncr));
			myDrawVertices[3].Set(-(archRadius+ringThickness)*cos(myGamma+i*angleIncr),(archRadius+ringThickness)*sin(myGamma+i*angleIncr));
			m_debugDraw.DrawPolygon(myDrawVertices,4, b2Color(1.0f, 0.0f, 0.0f));
		}
#endif
		//save output and update bodyTotRot
		if(startTest){//do in every time step during the test stage
			outputStream[1]<<endl<<myTime<<","<<loadingForce/1000<<","<<loadPointDispl.Length();//save output 

			for (int i=0; i<noPBodies; ++i){//update bodyTotRot
				float rotIncr = timeStepSize * myParticle[i]->GetAngularVelocity();
				bodyTotRotP[i] += rotIncr;
			}
		}

		//save geometry
		if (startTest && loadPointDispl.Length()>saveTargetDispl){	
			saveTargetDispl+=saveStepDispl;
			saveQSim();//save simulation data (x,y,rotation) of all quadrilaterals to the relevant output files
			savePSim();//save simulation data (x,y,rotation, total rotation) of all dodecagon particles to the relevant output files
			++noTS;	
		} 	

		//saveData and end test
		if (loadPointDispl.Length()>ringThickness){
#ifdef _WIN32
			stop_cpuWindows = cpuTimer();
			measuredCPUTime = stop_cpuWindows-start_cpuWindows;
#endif
#ifdef __unix__
			elapsed_cpuUNIX = clock() - start_cpuUNIX;
			measuredCPUTime = ((double)elapsed_cpuUNIX)/CLOCKS_PER_SEC;
#endif
			outputStream[0]<<endl<<loadingForce/1000<<","<<backfillUnitWeight<<","<<myTime<<","<<measuredCPUTime<<","
				<<span<<","<<spanToRise<<","<<ringThickness<<","<<noBricks<<","<<myFriction<<","<<loadingPos<<","<<masonryUnitWeight
				<<","<<partIniFri<<","<<circRH<<","<<particleUnitWeight<<","<<fillHeight<<","<<lBoundOffset<<","<<rBoundOffset
				<<","<<testGrav <<","<<maxLoadRate<<","<<randSeed<<","<<timeStepSize<<","<<velocityIterations;
			outputStream[0].close();
			saveMeta();
			outputStream[2].close();//close metaData file
			run_simulation=false;
		}

    }//END RUN_SIMULATION
#ifdef GUI
    static Test* Create()
    {
        return new michal_backfilled_arch;
    }
#endif
#ifndef GUI
	cout<<"End of the test. CPU time: "<<measuredCPUTime<<"s"<<endl;
	return(0);
#endif
};

#endif

