// Asymptote file writer created by Michal Pytlos, May 2014

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

// see Croft A., Davison R, 'Mathematics for engineers', page 531
// Variable names must be known at a compile time - use arrays to represent multiple bodies
#ifndef COMPUTER_GRAPHICS_H
#define COMPUTER_GRAPHICS_H

#include <iostream>
using namespace std;
#include <vector>
using std::vector;
#include <math.h>

#define myPI 3.14159265358979323846f
#ifndef DEGTORAD /* this macro changes degrees to radians and vice-versa */
#define DEGTORAD 0.0174532925199432957f
#define RADTODEG 57.295779513082320876f
#endif

struct myVector
{
	float x;
	float y;

	// Add a vector to this vector; use this to change coords of a vertex from local to global reference system; this is operator overloading;
	//pass argument by reference - any changes made to the reference are passed through to the argument; you can pass by constant reference if you don't want to change the argument 
	//const reference is read-only variable
	//non-const reference is read-write variable
	void operator += (const myVector& v)
	{
		x += v.x; y += v.y;
	}

	void operator -= (const myVector& v)
	{
		x -= v.x; y -= v.y;
	}

	//calculate vector length
	float Length () const
	{
		return sqrt(x*x+y*y);
	}
	
};

myVector subtractVectors (const myVector& v1, const myVector& v2);
myVector rotateVector (float angle, const myVector& v); //angles in radians; rotate anticlockwise

struct myBody
{
	//vertices; resize to number of vertices (add one for center of mass) when creating myBody
	vector <myVector> vert;
	// values from time step chosen 
	myVector originPD;//point of origin in world coords 
	float angleD;
	//values from target time step chosen for displacement analysis
	myVector originPT;
	float angleT;
	myVector tCM;//center of mass at the stop time step (aka target)
	myVector dCM;//center of mass at the start time step (aka d)
	float radius; //used only for circles
};

#endif
