// Asymptote file writer created by Michal Pytlos, May 2014

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#include "drawingWriter.h"
#include "ComputerGraphics.h"
using namespace std;

//common variables - put them in the main and call fuinction by 
ofstream outputStream1;
float maxTotRot;//in radians
float maxTotDispl;
float maxDisplX;
float maxDisplY;
vector<int> myColourLayer;
vector<float> myDisplX; //displacement of a particle in the x direction 
vector<float> myDisplY; //displacement of a particle in the y direction 
vector<float> myTotDispl; //total displacement of a particle

void colLayersF(globalVars &myGV){
	int lColArray [2] = {8500, 32000};
	myColourLayer.resize(myGV.noPart); //RESIZE DO NOT DECLARE
	for (int partNo=0; partNo<myGV.noPart; ++partNo)
	{
		myColourLayer[partNo]=lColArray[1]; //default colour
		for (int i = 0; i<24; ++i)//divide soil height = 60 into 24 layers
		{
			if (myGV.mySimArrayP[1][0][partNo] < (i+1)*2.5){
				myColourLayer[partNo]=lColArray[i % 2];
				break;
			}
		}
	}
}

void displColF(globalVars &myGV){
		myDisplX.resize(myGV.noPart); //displacement of a particle in the x direction //RESIZE DO NOT DECLARE
		myDisplY.resize(myGV.noPart); //displacement of a particle in the y direction //RESIZE DO NOT DECLARE
		myTotDispl.resize(myGV.noPart); //total displacement of a particle //RESIZE DO NOT DECLARE
		myVector tempPart;
		 
		for (int partNo=0; partNo<myGV.noPart; ++partNo)
		{
			myDisplX[partNo]=myGV.part[partNo].tCM.x-myGV.part[partNo].dCM.x;
			myDisplY[partNo]=myGV.part[partNo].tCM.y-myGV.part[partNo].dCM.y;
			tempPart.x=myDisplX[partNo];
			tempPart.y=myDisplY[partNo];
			myTotDispl[partNo]=tempPart.Length();	
		}

		//calculate maximum total displacement for all particles
		maxTotDispl=0;
		maxDisplX=0;
		maxDisplY=0;
		for (int partNo=0; partNo<myGV.noPart; ++partNo)
		{
			if (abs(myDisplX[partNo])>maxDisplX){maxDisplX=abs(myDisplX[partNo]);}
			if (abs(myDisplY[partNo])>maxDisplY){maxDisplY=abs(myDisplY[partNo]);}
			if (abs(myTotDispl[partNo])>maxTotDispl){maxTotDispl=abs(myTotDispl[partNo]);}
		}
}

void iniAsyF(globalVars &myGV){
	
	string fileName=outputDir;
	fileName.append(myGV.identifier);
	ostringstream converter1, converter2;

	converter1<<myGV.timeStepD;
	converter2<<myGV.timeStepT;
	fileName.append("Asy_");
	fileName.append(converter1.str());
	fileName.append("_");
	fileName.append(converter2.str());
	if (myGV.drawEG==1){fileName.append("_EG");}//EG=end geometry
	fileName.append(".asy");
	cout<<"File name: "<<fileName<<endl;

	//DISPLACEMENT ANALYSIS
	int geomTS;//geometry time step
	if (myGV.drawEG==1){geomTS=myGV.timeStepT;}
	else {geomTS=myGV.timeStepD;}
	outputStream1.open(fileName);
	outputStream1<<"//Geometry drawn for time step: "<<geomTS<<endl<<"//Rotation and displacement drawn for time step: "<<myGV.timeStepT<<endl<<endl
		<<"bool drawVec=false;"<<endl<<"bool colLayer=false;"<<endl<<"bool rotFill=false;"<<endl<<"bool rotGrayFillW=false; //white=max rotation"<<endl<<"bool rotGrayFillB=false; //black=max rotation"<<endl
		<<"bool totDisplFill=false;"<<endl<<"bool xDisplFill=false;"<<endl<<"bool yDisplFill=false;"<<endl<<"real drawLim = "<<myGV.heightStaticPart<<"; //do not draw particles with position above this height"<<endl<<endl
		<<"size(15cm);"<<endl<<"import palette;"<<endl<<"import graph;"<<endl
		<<"if (rotGrayFillW || rotGrayFillB) {defaultpen(0.4mm);} //line thickness for pallete bar"<<endl<<"else {defaultpen(0.1mm);}"<<endl
		<<"pen[] palettePen=Rainbow();"<<endl<<"pen[] gPalette=Grayscale();"<<endl<<"pen fillPen=grey+0.1mm;"<<endl<<"pen fillPenQ=grey+0.1mm;"<<endl<<"pen boundPen=black+0.1mm;"<<endl
		<<"pen arrowPen=blue+0.15mm;"<<endl<<"real arrowHS=0.75mm;"<<endl<<"pen voidRatioPen=red+0.25mm;"<<endl<<"picture pic2;"<<endl<<"size (pic2, 8cm);"<<endl<<endl
		<<"void drawParticle(int layerCol, int rotCol, int rotGray, int totDisplCol, int xDisplCol, int yDisplCol, real x0, real y0, real x1, real y1, real x2, real y2, real x3, real y3, real x4, real y4, real x5, real y5, real x6, real y6, real x7, real y7, real x8, real y8, real x9, real y9, real x10, real y10, real x11, real y11)"<<endl
		<<"{"<<endl<<"if (y0<drawLim){"<<endl
		<<"	if (rotFill){filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, palettePen[rotCol], boundPen);}"<<endl
		<<"	else if (rotGrayFillW){filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, gPalette[rotGray], boundPen);}"<<endl
		<<"	else if (rotGrayFillB){filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, gPalette[255-rotGray], boundPen);}"<<endl
		<<"	else if (totDisplFill){filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, palettePen[totDisplCol], boundPen);}"<<endl
		<<"	else if (xDisplFill){filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, palettePen[xDisplCol], boundPen);}"<<endl
		<<"	else if (yDisplFill){filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, palettePen[yDisplCol], boundPen);}"<<endl
		<<"	else if (colLayer){filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, palettePen[layerCol], boundPen);}"<<endl
		<<"	else {filldraw((x0,y0)--(x1,y1)--(x2,y2)--(x3,y3)--(x4,y4)--(x5,y5)--(x6,y6)--(x7,y7)--(x8,y8)--(x9,y9)--(x10,y10)--(x11,y11)--cycle, fillPen, boundPen);}"<<endl<<"}"<<endl<<"}"<<endl<<endl	
		<<"void drawVector (real x0, real y0, real x1, real y1)"<<endl<<"{"<<endl
		<<"	if (drawVec) {draw((x0,y0)--(x1,y1), arrowPen, Arrow(arrowHS));}"<<endl<<"	else{}"<<endl<<"}"<<endl<<endl; 
}

void drawQF(globalVars &myGV){
	outputStream1<<"//Draw quadrilaterals"<<endl; 
	for (int quadNo=0; quadNo <myGV.noQuad; ++quadNo)
	{
		outputStream1<<"filldraw(";
		for (int i=0;i<(noCrdQ-1)/2;++i){
			outputStream1<<"("<<myGV.quad[quadNo].vert[i].x<<","<<myGV.quad[quadNo].vert[i].y<<")--";
		}
		outputStream1<<"cycle, fillPenQ, boundPen);"<<endl;
	}
}

void drawCirc(globalVars &myGV){
	outputStream1<<"//Draw circles"<<endl; 
	for (int circNo=0; circNo <myGV.noCirc; ++circNo){
		outputStream1<<"filldraw(circle(("<<myGV.circ[circNo].originPD.x<<","<<myGV.circ[circNo].originPD.y<<"),"<<myGV.circ[circNo].radius<<"), fillPen, boundPen);"<<endl;
	}
}

void drawEdge(globalVars &myGV){
	//open the geometry EGeometry.txt
		char buff [buffSize];
		ifstream inputStreamEG; 
		stringstream mySS;

		string inputFileName = inputDir;
		inputFileName.append(myGV.identifier);
		inputFileName.append("EGeometry.txt");
		inputStreamEG.open(inputFileName);

	//read coordinates and draw edges to the asy file
		outputStream1<<"//Draw an edge"<<endl;
		for (int i=0; i<myGV.noEdge; ++i){
			inputStreamEG.getline (buff, 1000);//read line
			mySS << buff;
			mySS.getline (buff, 1000, ',');
			float x0=atof(buff);
			mySS.getline (buff, 1000, ',');
			float x1=atof(buff);
			mySS.getline (buff, 1000, ',');
			float y0=atof(buff);
			mySS.getline (buff, 1000, ',');
			float y1=atof(buff);
			mySS <<"";
			mySS.clear(); 
			outputStream1<<"draw(("<<x0<<","<<y0<<")--("<<x1<<","<<y1<<"), boundPen);"<<endl;//draw an edge
		}
}

void rotColF (globalVars &myGV){
	maxTotRot=0;
	
	vector<float>myRot;
	myRot.resize(myGV.noPart);
	for (int partNo=0; partNo<myGV.noPart; ++partNo){//store all rotations in a vector
		myRot[partNo]=abs(myGV.mySimArrayP[3][myGV.timeStepT][partNo]-myGV.mySimArrayP[3][myGV.timeStepD][partNo]);
	}
	sort(myRot.begin(),myRot.end()); //sort vector 
	int maxRotPart = 0.99*myGV.noPart-1;//calculate maximum total rotation for all particles, ignore top 1% of particles
	maxTotRot=myRot[maxRotPart];
}

void drawPF(globalVars &myGV){
	outputStream1<<"//Draw particles"<<endl;
	for (int partNo=0; partNo<myGV.noPart; ++partNo)
	{	//colour for particles
		//take absolute value; make sure the result is integer and is equal or lower to 32765 (32766=number of colours); use colours from dark blue (7500) to red (32765)
		int rotCol=7500+abs((25265/maxTotRot)*(myGV.mySimArrayP[3][myGV.timeStepT][partNo]-myGV.mySimArrayP[3][myGV.timeStepD][partNo]));
		if (rotCol>32765){rotCol=32765;}
		int rotGray = abs((256/maxTotRot)*(myGV.mySimArrayP[3][myGV.timeStepT][partNo]-myGV.mySimArrayP[3][myGV.timeStepD][partNo]));
		if (rotGray>255){rotGray=255;}
		int totDisplCol=7500+abs((25265/maxTotDispl)*myTotDispl[partNo]);
		int xDisplCol=7500+abs((25265/maxDisplX)*myDisplX[partNo]);
		int yDisplCol=7500+abs((25265/maxDisplY)*myDisplY[partNo]);
		//outputStream1<<"currentpen=palettePen["<<rotCol<<"];"<<endl; 
		outputStream1<<"drawParticle("<<myColourLayer[partNo]<<","<<rotCol<<","<<rotGray<<","<<totDisplCol<<","<<xDisplCol<<","<<yDisplCol;
		for (int i=0;i<(noCrdP-1)/2;++i)
		{
			outputStream1<<","<<myGV.part[partNo].vert[i].x<<","<<myGV.part[partNo].vert[i].y;
		}
		outputStream1<<");"<<endl;
	}
}

void localVoidRatio(globalVars &myGV){// local void ratio calculations
	
	//3. (1) and (2) ask only if more than 0 particles; also ask user with 0 and 1 question whether wants this function (and vector non default value)
	//5. Loop thorygh all particles if inside zone add volume to the volume pool

	float minX=1000;
	float minY=1000;
	float maxX=-1000;
	float maxY=-1000;
	myVector voidRatioZoneCenter;
	myVector voidRatioPartDist;//distance of a particle from the center of the void ratio zone
	float tempPartVolume; //volume of the current particle
	int noPartVoidRatio = 0;//number of particles inside the void ratio zone 
	float partTotVolume = 0;//total volume of particles within the void ratio zone
	float voidRatio;

	//1. Sort through all particles to find region populated with non-static particles
	for (int partNo=0; partNo<myGV.noPart; ++partNo){
		if (myGV.part[partNo].vert[12].y<myGV.heightStaticPart){
			if(myGV.part[partNo].vert[12].x<minX){minX=myGV.part[partNo].vert[12].x;}//vert[12] is center of mass; location is the same as in the drawing  
			if(myGV.part[partNo].vert[12].y<minY){minY=myGV.part[partNo].vert[12].y;}
			if(myGV.part[partNo].vert[12].x>maxX){maxX=myGV.part[partNo].vert[12].x;}
			if(myGV.part[partNo].vert[12].y>maxY){maxY=myGV.part[partNo].vert[12].y;}
		}
	}
	//2. Calculate center of the void ratio zone
	voidRatioZoneCenter.x = minX+myGV.voidRatioZoneX*(maxX-minX);
	voidRatioZoneCenter.y = minY+myGV.voidRatioZoneY*(maxY-minY);

	//3. Draw void ratio zone 
	outputStream1<<"//Draw void ratio zone"<<endl
		<<"draw(Circle(("<<voidRatioZoneCenter.x<<","<<voidRatioZoneCenter.y<<"),"<<myGV.voidRatioZoneRad<<"),voidRatioPen);"<<endl;

	//4. Draw region populated with non-static particles (optional; off by default)
	outputStream1<<"//Draw region populated with non-static particles"<<endl
		<<"draw(("<<minX<<","<<minY<<")--("<<maxX<<","<<minY<<")--("<<maxX<<","<<maxY<<")--("<<minX<<","<<maxY<<")--cycle,voidRatioPen);"<<endl;

	//5. Loop through all particles -> check if inside void ratio zone0 ->  if (yes) then add volume to the volume pool 
	for (int partNo=0; partNo<myGV.noPart; ++partNo){

		voidRatioPartDist=voidRatioZoneCenter;
		voidRatioPartDist -= myGV.part[partNo].vert[12];

		if (voidRatioPartDist.Length()<myGV.voidRatioZoneRad){//add volume
			tempPartVolume=0;//for theory on volume calcs see: https://en.wikipedia.org/wiki/Shoelace_formula
			for (int i=0; i<11; ++i){
				tempPartVolume+=(myGV.part[partNo].vert[i].x*myGV.part[partNo].vert[i+1].y);
			}
			tempPartVolume+=(myGV.part[partNo].vert[11].x*myGV.part[partNo].vert[0].y);
				
			for (int i=0; i<11; ++i){
				tempPartVolume-=(myGV.part[partNo].vert[i+1].x*myGV.part[partNo].vert[i].y);
			}
			tempPartVolume-=(myGV.part[partNo].vert[0].x*myGV.part[partNo].vert[11].y);
			tempPartVolume=0.5*abs(tempPartVolume);	
			partTotVolume+=tempPartVolume;
			++noPartVoidRatio;
		}
	}
	//6. Print the results
	voidRatio = (myPI*myGV.voidRatioZoneRad*myGV.voidRatioZoneRad-partTotVolume)/partTotVolume;
	outputStream1<<"//Void ratio: "<<voidRatio<<endl
		<<"//Number of particles within the void ratio zone: "<<noPartVoidRatio<<endl;
}

void drawVecQ(globalVars &myGV){
	//draw vectors 
	outputStream1<<"//Draw vectors for quadrilaterals"<<endl;
	for (int quadNo=0; quadNo <myGV.noQuad; ++quadNo)
	{
		outputStream1<<"drawVector("<<myGV.quad[quadNo].vert[4].x<<","<<myGV.quad[quadNo].vert[4].y<<","<<myGV.quad[quadNo].tCM.x<<","<<myGV.quad[quadNo].tCM.y<<");"<<endl;
	}
}

void drawVecC(globalVars &myGV){
	//draw vectors 
	outputStream1<<"//Draw vectors for circles"<<endl;
	for (int circNo=0; circNo <myGV.noCirc; ++circNo){
		outputStream1<<"drawVector("<<myGV.circ[circNo].originPD.x<<","<<myGV.circ[circNo].originPD.y<<","<<myGV.circ[circNo].originPT.x<<","<<myGV.circ[circNo].originPT.y<<");"<<endl;
	}
}

void drawVecP(globalVars &myGV){
	outputStream1<<"//Draw vectors for particles"<<endl;
	//DRAW VECTORS
	float minX=1000;
	float minY=1000;
	float maxX=-1000;
	float maxY=-1000;
	myVector partCM;
	myVector distV;
	float dist;
	float minDist;
	myVector gridPoint[maxGRCNo][maxGRCNo]; 
	int vecPartNo;
	//1. Sort thorugh all particles to find region populated with particles
	for (int partNo=0; partNo<myGV.noPart; ++partNo)
	{
		if(myGV.part[partNo].vert[12].x<minX){minX=myGV.part[partNo].vert[12].x;}
		if(myGV.part[partNo].vert[12].y<minY){minY=myGV.part[partNo].vert[12].y;}
		if(myGV.part[partNo].vert[12].x>maxX){maxX=myGV.part[partNo].vert[12].x;}
		if(myGV.part[partNo].vert[12].y>maxY){maxY=myGV.part[partNo].vert[12].y;}
	}

	//2. Create grid
	int gridNoRow=(maxY-minY)/myGV.gridInterval;
	int gridNoCol=(maxX-minX)/myGV.gridInterval;
	float cWidth = minX;//current width
	float cHeight = minY;

	for (int myRow=0; myRow<gridNoRow; ++myRow)
	{
		for(int myCol=0; myCol<gridNoCol; ++myCol)
		{
			gridPoint[myRow][myCol].x=cWidth;
			gridPoint[myRow][myCol].y=cHeight;
			cWidth+=myGV.gridInterval;
		}
		cWidth=minX;
		cHeight+=myGV.gridInterval;
	}

	//3. Find particles closest to the grid points and draw vectors for them
	for (int myRow=0;myRow<gridNoRow;++myRow)
	{
		for (int myCol=0;myCol<gridNoCol;++myCol)
		{
			minDist=1000;
			for (int partNo=0; partNo<myGV.noPart; ++partNo)
			{
				partCM.x=myGV.part[partNo].vert[12].x;
				partCM.y=myGV.part[partNo].vert[12].y;
				distV=subtractVectors(gridPoint[myRow][myCol],partCM);
				dist=distV.Length();
				if (dist<minDist)
				{
					minDist=dist; 
					vecPartNo=partNo;
				}
			}
			
			//draw vector only if it's close enough to the grid point
			if (minDist<myGV.gridInterval) {outputStream1<<"drawVector("<<myGV.part[vecPartNo].vert[12].x<<","<<myGV.part[vecPartNo].vert[12].y<<","<<myGV.part[vecPartNo].tCM.x<<","<<myGV.part[vecPartNo].tCM.y<<");"<<endl;}
		}
	}
}

void paletteBar(globalVars &myGV){
	//palette bar
	int maxTotRotI = maxTotRot*RADTODEG;
	cout<<"maxTotRot: "<<maxTotRot;
	outputStream1<<endl<<"//Draw palette bar"<<endl
		<<"int maxTotRot="<<maxTotRotI<<";"<<endl<<"real maxTotDispl="<<maxTotDispl<<";"<<endl<<"real maxDisplX="<<maxDisplX<<";"<<endl<<"real maxDisplY="<<maxDisplY<<";"<<endl
		<<"int n=25266;"<<endl<<"int n2=255;"<<endl//25266=32766-7500
	<<"if(rotFill){"<<endl
		<<"	for (int i=0; i<n; i+=1) {currentpen=palettePen[7500+i]; draw(pic2,(i*maxTotRot/n,0)--(i*maxTotRot/n,0.06*maxTotRot));}"<<endl
		<<"	xaxis(pic2,scale(0.7)*\"$Rotation\\\:[^{\\\circ}]$\", Bottom, 0, maxTotRot, boundPen,RightTicks(scale(0.5)*rotate(90)*Label(),step=10,Step=20),above=true);"<<endl<<"}"<<endl
	<<"if(rotGrayFillW){"<<endl
		<<"	for (int i=0; i<n2; i+=1) {currentpen=gPalette[i]; draw(pic2,(i*maxTotRot/n2,0)--(i*maxTotRot/n2,0.06*maxTotRot));}"<<endl
		<<"	xaxis(pic2,scale(0.7)*\"$Rotation\\\:[^{\\\circ}]$\", Bottom, 0, maxTotRot, boundPen,RightTicks(scale(0.5)*rotate(90)*Label(),step=10,Step=20),above=true);"<<endl<<"}"<<endl
	<<"if(rotGrayFillB){"<<endl
		<<"	for (int i=0; i<n2; i+=1) {currentpen=gPalette[255-i]; draw(pic2,(i*maxTotRot/n2,0)--(i*maxTotRot/n2,0.06*maxTotRot));}"<<endl
		<<"	xaxis(pic2,scale(0.7)*\"$Rotation\\\:[^{\\\circ}]$\", Bottom, 0, maxTotRot, boundPen,RightTicks(scale(0.5)*rotate(90)*Label(),step=10,Step=20),above=true);"<<endl<<"}"<<endl
	<<"if(totDisplFill){"<<endl
		<<"	for (int i=0; i<n; i+=1) {currentpen=palettePen[7500+i]; draw(pic2,(i*maxTotDispl/n,0)--(i*maxTotDispl/n,0.06*maxTotDispl));}"<<endl
		<<"	xaxis(pic2,scale(0.7)*\"$Total \\\: Displacement$\", Bottom, 0, maxTotDispl, boundPen,RightTicks(scale(0.5)*rotate(90)*Label(),step=0.2,Step=1),above=true);"<<endl<<"}"<<endl
	<<"if(xDisplFill){"<<endl
		<<"	for (int i=0; i<n; i+=1) {currentpen=palettePen[7500+i]; draw(pic2,(i*maxDisplX/n,0)--(i*maxDisplX/n,0.06*maxDisplX));}"<<endl
		<<"	xaxis(pic2,scale(0.7)*\"$Displacement \\\: x$\", Bottom, 0, maxDisplX, boundPen,RightTicks(scale(0.5)*rotate(90)*Label(),step=0.2,Step=1),above=true);"<<endl<<"}"<<endl
	<<"if(yDisplFill){"<<endl
		<<"	for (int i=0; i<n; i+=1) {currentpen=palettePen[7500+i]; draw(pic2,(i*maxDisplY/n,0)--(i*maxDisplY/n,0.06*maxDisplY));}"<<endl
		<<"	xaxis(pic2,scale(0.7)*\"$Displacement \\\: y$\", Bottom, 0, maxDisplY, boundPen,RightTicks(scale(0.5)*rotate(90)*Label(),step=0.2,Step=1),above=true);"<<endl<<"}"<<endl
		<<"frame f2=pic2.fit();"<<endl<<"add(f2, (-55,-20));";
	
	outputStream1.close();
}