// Asymptote file writer created by Michal Pytlos, May 2014

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#include "drawingWriter.h"
#include "ComputerGraphics.h"
using namespace std;

void myCircle (globalVars &myGV) {

	//DECLARE AND RESIZE VECTORS
	//1D vector 
	vector<float> myGeomArrayC;
	myGeomArrayC.resize(arraySizeP);

	//3D vector
	vector<vector<vector<float> > > mySimArrayC;
	mySimArrayC.resize(arraySizeI);
	for (int i = 0; i < arraySizeI; ++i)
	{
		mySimArrayC[i].resize(arraySizeT);

		for (int j = 0; j < arraySizeT; ++j){
			mySimArrayC[i][j].resize(arraySizeP);
		}
	}

	//GEOMETRY ARRAY
	char buff [buffSize];
	ifstream inputStreamCG; 
	stringstream mySS;

	string inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("CGeometry.txt");
	inputStreamCG.open(inputFileName);

	//construct geometry array - radius of each circle
	for (int myRow = 0; myRow < myGV.noCirc; ++myRow)
	{
		inputStreamCG.getline (buff, buffSize);
		mySS << buff;
		mySS.getline (buff, 1000, ','); 
		myGeomArrayC [myRow] = atof (buff); 
		mySS <<"";
		mySS.clear(); 
	}

	//SIMULATION ARRAY
	ifstream inputStreamC[2]; 
	//open file to read from
	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("CGlobalX.txt");
	inputStreamC[0].open(inputFileName);

	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("CGlobalY.txt");
	inputStreamC[1].open(inputFileName);

	//construct simulation array
	for (int myDepth = 0; myDepth < noInpC; ++myDepth)//depths correspond to coords of point of origin (x,y) and rotation
	{
		for (int myRow = 0; myRow < myGV.noTS; ++myRow)//rows correspond to time steps 
		{
			inputStreamC[myDepth].getline (buff, buffSize);
			mySS << buff;

			for (int myCol=0; myCol < myGV.noCirc; ++myCol){//columns correspond to circles
				mySS.getline (buff, 1000, ',');
				mySimArrayC [myDepth][myRow][myCol] = atof (buff);
			}
			mySS <<"";
			mySS.clear();
		}
	}

	//BUILD GEOMETRY STRUCTURES FROM GEOMETRY ARRAY AND SIMULATION ARRAY
	myGV.circ.resize(arraySizeP);
	
	for (int circNo=0; circNo<myGV.noCirc; ++circNo){
		myGV.circ[circNo].originPD.x=mySimArrayC[0][myGV.timeStepD][circNo];
		myGV.circ[circNo].originPD.y=mySimArrayC[1][myGV.timeStepD][circNo];
		myGV.circ[circNo].originPT.x=mySimArrayC[0][myGV.timeStepT][circNo];
		myGV.circ[circNo].originPT.y=mySimArrayC[1][myGV.timeStepT][circNo];
		myGV.circ[circNo].radius=myGeomArrayC[circNo];
	}
}