// Asymptote file writer created by Michal Pytlos, May 2014

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#include "drawingWriter.h"
#include "ComputerGraphics.h"
using namespace std;

void myDodecagon (globalVars &myGV) {	

	//initialize 2D vector array 
	vector< vector<float> > myGeomArrayP;
	myGeomArrayP.resize(arraySizeP);
	for (int i = 0; i < arraySizeP; ++i){
		myGeomArrayP[i].resize(arraySizeC);
	}

	//initialize 3D vector array
	myGV.mySimArrayP.resize(arraySizeI);
	for (int i = 0; i < arraySizeI; ++i)
	{
		myGV.mySimArrayP[i].resize(arraySizeT);

		for (int j = 0; j < arraySizeT; ++j){
			myGV.mySimArrayP[i][j].resize(arraySizeP);
		}
	}

	//GEOMETRY ARRAY
	char buff [buffSize];
	ifstream inputStreamPG; 
	stringstream mySS;
	
	string inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("PGeometry.txt");
	inputStreamPG.open(inputFileName);
	for (int myRow = 0; myRow < myGV.noPart; ++myRow)//each row corresponds to one particle
	{
		inputStreamPG.getline (buff, buffSize);
		mySS << buff;
		//seperate columns from the string of data
		for (int myCol=0; myCol < noCrdP; ++myCol)//columns correspond to coords
		{		
			mySS.getline (buff, 1000, ','); 
			myGeomArrayP [myRow][myCol] = atof (buff); 
		}
		mySS <<"";	
		mySS.clear(); 
	}

	//SIMULATION ARRAY
	ifstream inputStreamP[4]; 
	//open file to read from
	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("PGlobalX.txt");
	inputStreamP[0].open(inputFileName);

	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("PGlobalY.txt");
	inputStreamP[1].open(inputFileName);

	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("PRotation.txt");
	inputStreamP[2].open(inputFileName);

	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("PTotRot.txt");
	inputStreamP[3].open(inputFileName);
	//construct simulation array
	for (int myDepth = 0; myDepth < noInpP; ++myDepth)//depths correspond to coords of point of origin (x,y) and rotation
	{
		for (int myRow = 0; myRow < myGV.noTS; ++myRow)//rows correspond to time steps 
		{
			inputStreamP[myDepth].getline (buff, buffSize);
			mySS << buff;

			for (int myCol=0; myCol < myGV.noPart; ++myCol)//columns correspond to particles
			{
				mySS.getline (buff, 1000, ',');
				myGV.mySimArrayP [myDepth][myRow][myCol] = atof (buff);
			}
			mySS <<"";
			mySS.clear();
		}
	}

	//BUILD GEOMETRY STRUCTURES FROM GEOMETRY ARRAY AND SIMULATION ARRAY
	myGV.part.resize(arraySizeP);
	
	for (int partNo=0; partNo<myGV.noPart; ++partNo)
	{
		myGV.part[partNo].vert.resize(13);
		for (int i=0; i<noCrdP/2; ++i){
			myGV.part[partNo].vert[i].x=myGeomArrayP[partNo][i];
			myGV.part[partNo].vert[i].y=myGeomArrayP[partNo][i+noCrdP/2];
		}
		myGV.part[partNo].originPD.x=myGV.mySimArrayP[0][myGV.timeStepD][partNo];
		myGV.part[partNo].originPD.y=myGV.mySimArrayP[1][myGV.timeStepD][partNo];
		myGV.part[partNo].angleD=myGV.mySimArrayP[2][myGV.timeStepD][partNo];
		//displacement analsysis - target
		myGV.part[partNo].originPT.x=myGV.mySimArrayP[0][myGV.timeStepT][partNo];
		myGV.part[partNo].originPT.y=myGV.mySimArrayP[1][myGV.timeStepT][partNo];
		myGV.part[partNo].angleT=myGV.mySimArrayP[2][myGV.timeStepT][partNo];
	}

	//TRANSFORM EACH VERTEX
	float myAngle;
	myVector myOrigin;
	for (int partNo=0; partNo<myGV.noPart; ++partNo)
	{
		if (myGV.drawEG==1){
			myAngle = myGV.part[partNo].angleT;
			myOrigin = myGV.part[partNo].originPT;
		}
		else{
			myAngle = myGV.part[partNo].angleD;
			myOrigin = myGV.part[partNo].originPD;
		}
		//center of mass at the start time step
		myGV.part[partNo].dCM=rotateVector(myGV.part[partNo].angleD, myGV.part[partNo].vert[12]);
		myGV.part[partNo].dCM+=myGV.part[partNo].originPD;
		//center of mass at the stop time step [aka target]
		myGV.part[partNo].tCM=rotateVector(myGV.part[partNo].angleT, myGV.part[partNo].vert[12]);
		myGV.part[partNo].tCM+=myGV.part[partNo].originPT;
		for (int i=0; i<noCrdP/2; ++i)
		{
			//rotate by angle
			myGV.part[partNo].vert[i]=rotateVector(myAngle, myGV.part[partNo].vert[i]);
			//translate
			myGV.part[partNo].vert[i]+=myOrigin;
		}
	}
}