// Asymptote file writer created by Michal Pytlos, May 2014

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#include "drawingWriter.h"
#include "ComputerGraphics.h"
using namespace std;

int main (int argc, char** argv) {

	string inputFileName;
	globalVars myGV;

	//detect batch input
	bool batch_input = false;
	if (argc >= 2)	{batch_input = true;}

	//identify file name
	ifstream inputNameStream;
	char buff0[1000];
	int fileNo;

	if (batch_input) {fileNo = atoi(argv[1]);} //which test from the list do you want to produce .asy file for ? 
	else {cout<<"Choose test number (start from 1): "; cin>>fileNo;}

	inputFileName = inputDir;
	inputFileName.append("Identifiers.txt");
	inputNameStream.open(inputFileName);
	for (int i=0; i<fileNo; ++i){inputNameStream.getline (buff0, 1000);}
	myGV.identifier = buff0;
	cout<<"Identifier: "<<myGV.identifier<<endl<<endl;
	
	//read meta file
	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("metaData.txt");

	ifstream inputStream0; //object to operate on a file
	stringstream mySS0;
	
	inputStream0.open(inputFileName); //open file to read from, this will search for file in the default solution directory
	inputStream0.getline (buff0, 1000); //read full line from file into a buffer; getline allows to read an entire text line from any stream into a string variable // parameter 1: where to write line to; parameter 2 size of the array
	mySS0 << buff0; //copy the line into string stream (to speparate into column and convert to int)
	//number of quadrilaterals
	mySS0.getline (buff0, 1000, ',');//read from mySS into buffer; read only until delimiter is reached // parameter 1: where to write line to; paremeter 2: size of the array; parameter 3: delimiter
	myGV.noQuad = atof (buff0); //atof converts string/char to float;
	//number of particles
	mySS0.getline (buff0, 1000, ',');
	myGV.noPart = atof (buff0); 
	//number of circles
	mySS0.getline (buff0, 1000, ',');
	myGV.noCirc = atof (buff0); 
	//number of edges
	mySS0.getline (buff0, 1000, ',');
	myGV.noEdge = atof (buff0);
	//number of recorded time steps
	mySS0.getline (buff0, 1000, ',');
	myGV.noTS = atof (buff0);

	//REQUIRED INPUT
	if (batch_input)//TO_DO this is not up to date
	{
		myGV.timeStepD = atoi(argv[2]);
		myGV.timeStepT = atoi(argv[3]);
		myGV.drawEG = atoi(argv[4]);
		myGV.gridInterval = atof(argv[5]);
	}

	else //CONSOLE INPUT
	{
		cout<<"Number of quadrilaterals: "<<myGV.noQuad<<endl
			<<"Number of particles: "<<myGV.noPart<<endl
			<<"Number of recorded time steps: "<<myGV.noTS<<endl<<endl
			<<"Choose start time step (0-"<<myGV.noTS-1<<"): ";
		cin >> myGV.timeStepD;
		cout<<"Choose end time step (0-"<<myGV.noTS-1<<"): ";
		cin >> myGV.timeStepT;
		cout<<"Draw end geometry ? (1=true): ";
		cin >> myGV.drawEG;

		//void ratio calcs
		cout<<endl<<"Void ratio calcs ? (1=true): ";
		cin >> myGV.voidRatioCalcs;
		cout<<"Calculations are done for the drawn geometry."<<endl;
		
		if (myGV.voidRatioCalcs==1){
			cout<<"Specify void ratio zone radius: ";
			cin >> myGV.voidRatioZoneRad;
			cout<<"Specify void ratio zone center X (0=min; 1=max): ";
			cin >> myGV.voidRatioZoneX;
			cout<<"Specify void ratio zone center Y (0=min; 1=max): ";
			cin >> myGV.voidRatioZoneY;
		}
		
		//optional input
		cout<<endl<<"Additional user input ? (1=true): ";
		cin >> myGV.optionalInput;
		if (myGV.optionalInput==1){
			cout<<"Specify vector grid spacing: ";
			cin >> myGV.gridInterval;
			cout<<"Specify static particles height: ";
			cin >> myGV.heightStaticPart;
		}
		else{
			myGV.gridInterval=5;
			myGV.heightStaticPart=105;
			cout<<"Vector grid spacing: "<<myGV.gridInterval<<endl
				<<"Static particles height: "<<myGV.heightStaticPart<<endl;
		}	
	}

	//call functions
	if (myGV.noPart>0) {myDodecagon(myGV);}//run only if there are some dodecagons
	if (myGV.noQuad>0) {myQuadrilateral(myGV);}//run only if there are some quadrilaterals
	if (myGV.noCirc>0) {myCircle(myGV);}//run only if there are some circles
	if (myGV.noPart>0) {colLayersF (myGV);} 
	if (myGV.noPart>0) {displColF(myGV);} 
	iniAsyF(myGV);
	if (myGV.noQuad>0) {drawQF(myGV);} 
	if (myGV.noCirc>0) {drawCirc(myGV);} 
	if (myGV.noEdge>0) {drawEdge(myGV);} 
	if (myGV.noPart>0) {rotColF (myGV);}
	if (myGV.noPart>0) {drawPF(myGV);}
	if (myGV.noQuad>0) {drawVecQ(myGV);}
	if (myGV.noCirc>0) {drawVecC(myGV);}
	if (myGV.noPart>0) {drawVecP(myGV);}
	if (myGV.noPart>0 && myGV.voidRatioCalcs==1) {localVoidRatio(myGV);}
	if (myGV.noPart>0) {paletteBar(myGV);}
	

	return (0);
}