// Asymptote file writer created by Michal Pytlos, May 2014

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#ifndef DRAWING_WRITER_H
#define DRAWING_WRITER_H

#include <iostream>
#include <fstream> //class to read from file
#include <sstream> //stringstream
#include <string>
#include <cmath>
#include <stdlib.h>
#include <algorithm>
#include "ComputerGraphics.h"
using namespace std;

const string inputDir="../../../input/";
const string outputDir="../../../output/";

const int noCrdQ = 10;//number of coordinates = (number of vertices + 1)*2; +1 is for center of mass 
const int noCrdP = 26;
const int noInpC = 2;//number of input information for C simulation array; currently x,y
const int noInpQ = 3;//number of input information for Q simulation array; currently x,y and rotation angle
const int noInpP = 4;//number of input information for P simulation array; currently x,y,rotation angle and total rotation

//Array sizes must be bigger than corresponding values above
const int arraySizeQ = 1000;//number of quadrilaterals
const int arraySizeP = 20000;//number of particles or discs
const int arraySizeT = 250;//number of time steps
const int arraySizeC = 26;//number of coordinates
const int arraySizeI = 5;//number of inputs
const int buffSize = 200000;//this must be big
const int maxGRCNo = 50;//number of rows and columns in vector grid

struct globalVars{//global variables struct
	vector<vector<vector<float> > > mySimArrayP;
	myBody quad[arraySizeQ];
	vector<myBody> part;
	vector<myBody> circ;
	string identifier;
	int noQuad;
	int noPart;
	int noCirc;
	int noEdge;
	int noTS;
	int timeStepD, timeStepT;
	int drawEG; //draw end geometry; true if (drawEG==1); TO_DO currently works only with: dodecagon geometry, quad geometry, dodecagon rotations and dodecagon displacement
	
	//local void ratio calcs variables
	int voidRatioCalcs;//do void ratio calcs; true if (voidRatioCalcs==1)
	float voidRatioZoneRad;// radius of the zone for void ratio calcs
	float voidRatioZoneX;// x coord of the center of the zone; this is multiplied by the zone width 
	float voidRatioZoneY;// x coord of the center of the zone; this is multiplied by the zone height 

	//optional input
	int optionalInput; //true if (optionalInput==1)
	float gridInterval;
	float heightStaticPart;// particles above this height are considered to be static; they are not taken into account in the void ratio calcs; 	
};

void myDodecagon(globalVars &myGV); //dodecagon particles: create geometry and simulation arrays; create a geometry structure for each particle for choosen time steps
void myQuadrilateral(globalVars &myGV);//quadrilaterals: create geometry and simulation arrays; create a geometry structure for each quadrilatral for choosen time steps
void myCircle (globalVars &myGV);//circles: create geometry and simulation arrays; create a geometry structure for each quadrilatral for choosen time steps

//functions in asyFile.cpp
void colLayersF (globalVars &myGV); //assign a colour to each particle in order to create coloured layers; this is to help with visualization
void displColF(globalVars &myGV); //assign a colour to each particle based on its displacement; calculate max total displacement
void iniAsyF(globalVars &myGV);//create asy file and write definitions, comments etc. 
void drawQF(globalVars &myGV); //draw quadrilaterals
void drawCirc(globalVars &myGV); //draw circles
void drawEdge(globalVars &myGV); //draw static edges
void rotColF (globalVars &myGV);//assign a colour to each particle based on its rotation; calculate max total rotation
void drawPF(globalVars &myGV);// draw dodecagon particles
void drawVecQ(globalVars &myGV);//draw vectors for quadrilaterals
void drawVecC(globalVars &myGV);//draw vectors for circles
void drawVecP(globalVars &myGV);// draw vector for dodecagon particles
void paletteBar(globalVars &myGV);//draw pallete bars
void localVoidRatio(globalVars &myGV);// local void ratio calculations

#endif
