// Asymptote file writer created by Michal Pytlos, May 2014

//This file is a part of Appendix C of: Pytlos, M. (2015), New tools for modelling soil-filled masonry arch bridges, 
//PhD thesis, Department of Civil and Structural Engineering, University of Sheffield, Sheffield.
//For the copyright details please search for the thesis on: http://etheses.whiterose.ac.uk/ 
//In no event will the author be held liable for any damages arising from the use of this computer program.

#include "drawingWriter.h"
#include "ComputerGraphics.h"
using namespace std;

void myQuadrilateral (globalVars &myGV) 
{		
	//initialize 2D vector array (need to use heap memory because of stack overflow)
	vector< vector<float> > myGeomArrayQ;
	myGeomArrayQ.resize(arraySizeQ);
	for (int i = 0; i < arraySizeQ; ++i){
		myGeomArrayQ[i].resize(arraySizeC);
	}

	//initialize 3D vector array
	vector<vector<vector<float> > > mySimArrayQ;
	mySimArrayQ.resize(arraySizeI);
	for (int i = 0; i < arraySizeI; ++i)
	{
		mySimArrayQ[i].resize(arraySizeT);

		for (int j = 0; j < arraySizeT; ++j){
			mySimArrayQ[i][j].resize(arraySizeQ);
		}
	}

	//GEOMETRY ARRAY
	char buff [buffSize];
	ifstream inputStreamQG; 
	stringstream mySS;

	string inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("QGeometry.txt");
	inputStreamQG.open(inputFileName);

	//construct geometry array
	for (int myRow = 0; myRow < myGV.noQuad; ++myRow)//each row corresponds to one quadrilateral
	{
		inputStreamQG.getline (buff, buffSize);
		mySS << buff;

		//seperate columns from the string of data
		for (int myCol=0; myCol < noCrdQ; ++myCol)//columns correspond to coords
		{		
			mySS.getline (buff, 1000, ','); 
			myGeomArrayQ [myRow][myCol] = atof (buff); //save number to array; 
		}
		mySS <<"";	//clear the buffer (copy empty string to mySS erasing prevoius content)  
		mySS.clear(); //clear eof (end of file) flag 
	}

	//SIMULATION ARRAY
	ifstream inputStreamQ[3]; 
	//open file to read from
	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("QGlobalX.txt");
	inputStreamQ[0].open(inputFileName);

	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("QGlobalY.txt");
	inputStreamQ[1].open(inputFileName);

	inputFileName = inputDir;
	inputFileName.append(myGV.identifier);
	inputFileName.append("QRotation.txt");
	inputStreamQ[2].open(inputFileName);
	//construct simulation array
	for (int myDepth = 0; myDepth < noInpQ; ++myDepth)//depths correspond to coords of point of origin (x,y) and rotation
	{
		for (int myRow = 0; myRow < myGV.noTS; ++myRow)//rows correspond to time steps 
		{
			inputStreamQ[myDepth].getline (buff, buffSize);
			mySS << buff;

			for (int myCol=0; myCol < myGV.noQuad; ++myCol){//columns correspond to quadrilaterals
				mySS.getline (buff, 1000, ',');
				mySimArrayQ [myDepth][myRow][myCol] = atof (buff);
			}
			mySS <<"";
			mySS.clear();
		}
	}

	//BUILD GEOMETRY STRUCTURES FROM GEOMETRY ARRAY AND SIMULATION ARRAY
	for (int quadNo=0; quadNo<myGV.noQuad; ++quadNo)
	{
		myGV.quad[quadNo].vert.resize(5);
		for (int i=0; i<noCrdQ/2; ++i){
			myGV.quad[quadNo].vert[i].x=myGeomArrayQ[quadNo][i];
			myGV.quad[quadNo].vert[i].y=myGeomArrayQ[quadNo][i+noCrdQ/2];
		}
		//drawing
		myGV.quad[quadNo].originPD.x=mySimArrayQ[0][myGV.timeStepD][quadNo];
		myGV.quad[quadNo].originPD.y=mySimArrayQ[1][myGV.timeStepD][quadNo];
		myGV.quad[quadNo].angleD=mySimArrayQ[2][myGV.timeStepD][quadNo];
		//displacement analysis - target
		myGV.quad[quadNo].originPT.x=mySimArrayQ[0][myGV.timeStepT][quadNo];
		myGV.quad[quadNo].originPT.y=mySimArrayQ[1][myGV.timeStepT][quadNo];
		myGV.quad[quadNo].angleT=mySimArrayQ[2][myGV.timeStepT][quadNo];	
	}

	//TRANSFORM EACH VERTEX
	float myAngleQ;
	myVector myOriginQ;
	for (int quadNo=0; quadNo<myGV.noQuad; ++quadNo)
	{
		if (myGV.drawEG==1){
			myAngleQ = myGV.quad[quadNo].angleT;
			myOriginQ = myGV.quad[quadNo].originPT;
		}
		else{
			myAngleQ = myGV.quad[quadNo].angleD;
			myOriginQ = myGV.quad[quadNo].originPD;
		}

		//target center of mass
		myGV.quad[quadNo].tCM=rotateVector(myGV.quad[quadNo].angleT, myGV.quad[quadNo].vert[4]);
		myGV.quad[quadNo].tCM+=myGV.quad[quadNo].originPT;

		for (int i=0; i<noCrdQ/2; ++i){
			//rotate by angle
			myGV.quad[quadNo].vert[i]=rotateVector(myAngleQ, myGV.quad[quadNo].vert[i]);
			//translate
			myGV.quad[quadNo].vert[i]+=myOriginQ;
		}
	}
}

